# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import dashboard
from ..cog import variants as cogvariants


class Dashboard(cogbuilder.Builder[dashboard.Dashboard]):
    _internal: dashboard.Dashboard
    __current_y: int = 0
    __current_x: int = 0
    __last_panel_height: int = 0

    def __init__(self, title: str):
        self._internal = dashboard.Dashboard()        
        self._internal.title = title

    def build(self) -> dashboard.Dashboard:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: int) -> typing.Self:    
        """
        Unique numeric identifier for the dashboard.
        `id` is internal to a specific Grafana instance. `uid` should be used to identify a dashboard across Grafana instances.
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def uid(self, uid: str) -> typing.Self:    
        """
        Unique dashboard identifier that can be generated by anyone. string (8-40)
        """
            
        self._internal.uid = uid
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Title of dashboard.
        """
            
        self._internal.title = title
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of dashboard.
        """
            
        self._internal.description = description
    
        return self
    
    def revision(self, revision: int) -> typing.Self:    
        """
        This property should only be used in dashboards defined by plugins.  It is a quick check
        to see if the version has changed since the last time.
        """
            
        self._internal.revision = revision
    
        return self
    
    def gnet_id(self, gnet_id: str) -> typing.Self:    
        """
        ID of a dashboard imported from the https://grafana.com/grafana/dashboards/ portal
        """
            
        self._internal.gnet_id = gnet_id
    
        return self
    
    def tags(self, tags: list[str]) -> typing.Self:    
        """
        Tags associated with dashboard.
        """
            
        self._internal.tags = tags
    
        return self
    
    def timezone(self, timezone: str) -> typing.Self:    
        """
        Timezone of dashboard. Accepted values are IANA TZDB zone ID or "browser" or "utc".
        """
            
        self._internal.timezone = timezone
    
        return self
    
    def editable(self) -> typing.Self:    
        """
        Whether a dashboard is editable or not.
        """
            
        self._internal.editable = True
    
        return self
    
    def readonly(self) -> typing.Self:    
        """
        Whether a dashboard is editable or not.
        """
            
        self._internal.editable = False
    
        return self
    
    def tooltip(self, graph_tooltip: dashboard.DashboardCursorSync) -> typing.Self:    
        """
        Configuration of dashboard cursor sync behavior.
        Accepted values are 0 (sync turned off), 1 (shared crosshair), 2 (shared crosshair and tooltip).
        """
            
        self._internal.graph_tooltip = graph_tooltip
    
        return self
    
    def time(self, from_val: str, to: str) -> typing.Self:    
        """
        Time range for dashboard.
        Accepted values are relative time strings like {from: 'now-6h', to: 'now'} or absolute time strings like {from: '2020-07-10T08:00:00.000Z', to: '2020-07-10T14:00:00.000Z'}.
        """
            
        if self._internal.time is None:
            self._internal.time = dashboard.DashboardDashboardTime()
        assert isinstance(self._internal.time, dashboard.DashboardDashboardTime)
        self._internal.time.from_val = from_val    
        self._internal.time.to = to
    
        return self
    
    def timepicker(self, timepicker: cogbuilder.Builder[dashboard.TimePickerConfig]) -> typing.Self:    
        """
        Configuration of the time picker shown at the top of a dashboard.
        """
            
        timepicker_resource = timepicker.build()
        self._internal.timepicker = timepicker_resource
    
        return self
    
    def fiscal_year_start_month(self, fiscal_year_start_month: int) -> typing.Self:    
        """
        The month that the fiscal year starts on.  0 = January, 11 = December
        """
            
        if not fiscal_year_start_month < 12:
            raise ValueError("fiscal_year_start_month must be < 12")
        self._internal.fiscal_year_start_month = fiscal_year_start_month
    
        return self
    
    def live_now(self, live_now: bool) -> typing.Self:    
        """
        When set to true, the dashboard will redraw panels at an interval matching the pixel width.
        This will keep data "moving left" regardless of the query refresh rate. This setting helps
        avoid dashboards presenting stale live data
        """
            
        self._internal.live_now = live_now
    
        return self
    
    def week_start(self, week_start: str) -> typing.Self:    
        """
        Day when the week starts. Expressed by the name of the day in lowercase, e.g. "monday".
        """
            
        self._internal.week_start = week_start
    
        return self
    
    def refresh(self, refresh: typing.Union[str, typing.Literal[False]]) -> typing.Self:    
        """
        Refresh rate of dashboard. Represented via interval string, e.g. "5s", "1m", "1h", "1d".
        """
            
        self._internal.refresh = refresh
    
        return self
    
    def version(self, version: int) -> typing.Self:    
        """
        Version of the dashboard, incremented each time the dashboard is updated.
        """
            
        self._internal.version = version
    
        return self
    
    def with_panel(self, panel: cogbuilder.Builder[dashboard.Panel]) -> typing.Self:    
        if self._internal.panels is None:
            self._internal.panels = []
        
        panel_resource = panel.build()
        
        if panel_resource.grid_pos is None:
            panel_resource.grid_pos = dashboard.GridPos()
        
        # The panel either has no position set, or it is the first panel of the dashboard.
        # In that case, we position it on the grid
        if panel_resource.grid_pos.x == 0 and panel_resource.grid_pos.y == 0:
            panel_resource.grid_pos.x = self.__current_x
            panel_resource.grid_pos.y = self.__current_y
        self._internal.panels.append(panel_resource)
        
        # Prepare the coordinates for the next panel
        self.__current_x += panel_resource.grid_pos.w
        self.__last_panel_height = max(self.__last_panel_height, panel_resource.grid_pos.h)
        
        # Check for grid width overflow?
        if self.__current_x >= 24:
            self.__current_x = 0
            self.__current_y += self.__last_panel_height
            self.__last_panel_height = 0
    
        return self
    
    def with_row(self, row_panel: cogbuilder.Builder[dashboard.RowPanel]) -> typing.Self:    
        if self._internal.panels is None:
            self._internal.panels = []
        
        row_panel_resource = row_panel.build()
        
        # Position the row on the grid
        if row_panel_resource.grid_pos is None or (row_panel_resource.grid_pos.x == 0 and row_panel_resource.grid_pos.y == 0):
            row_panel_resource.grid_pos = dashboard.GridPos(
                x=0,
                y=self.__current_y + self.__last_panel_height,
                h=1,
                w=24,
            )
        self._internal.panels.append(row_panel_resource)
        
        # Reset the state for the next row
        self.__current_x = 0
        self.__current_y = row_panel_resource.grid_pos.y + 1
        self.__last_panel_height = 0
        
        # Position the row's panels on the grid
        for panel in row_panel_resource.panels:
            # Position the panel on the grid
            if panel.grid_pos is None:
                panel.grid_pos = dashboard.GridPos()
        
            # The panel either has no position set, or it is the first panel of the dashboard.
            # In that case, we position it on the grid
            if panel.grid_pos.x == 0 and panel.grid_pos.y == 0:
                panel.grid_pos.x = self.__current_x
                panel.grid_pos.y = self.__current_y
        
            # Prepare the coordinates for the next panel
            self.__current_x += panel.grid_pos.w
            self.__last_panel_height = max(self.__last_panel_height, panel.grid_pos.h)
        
            # Check for grid width overflow?
            if self.__current_x >= 24:
                self.__current_x = 0
                self.__current_y += self.__last_panel_height
                self.__last_panel_height = 0
    
        return self
    
    def variables(self, variables: list[cogbuilder.Builder[dashboard.VariableModel]]) -> typing.Self:    
        """
        Configured template variables
        """
            
        if self._internal.templating is None:
            self._internal.templating = dashboard.DashboardDashboardTemplating()
        assert isinstance(self._internal.templating, dashboard.DashboardDashboardTemplating)
        variables_resources = [r1.build() for r1 in variables]
        self._internal.templating.list_val = variables_resources
    
        return self
    
    def with_variable(self, variable: cogbuilder.Builder[dashboard.VariableModel]) -> typing.Self:    
        """
        Configured template variables
        """
            
        if self._internal.templating is None:
            self._internal.templating = dashboard.DashboardDashboardTemplating()
        assert isinstance(self._internal.templating, dashboard.DashboardDashboardTemplating)
        if self._internal.templating.list_val is None:
            self._internal.templating.list_val = []
        
        variable_resource = variable.build()
        self._internal.templating.list_val.append(variable_resource)
    
        return self
    
    def annotations(self, annotations: list[cogbuilder.Builder[dashboard.AnnotationQuery]]) -> typing.Self:    
        """
        Contains the list of annotations that are associated with the dashboard.
        Annotations are used to overlay event markers and overlay event tags on graphs.
        Grafana comes with a native annotation store and the ability to add annotation events directly from the graph panel or via the HTTP API.
        See https://grafana.com/docs/grafana/latest/dashboards/build-dashboards/annotate-visualizations/
        """
            
        if self._internal.annotations is None:
            self._internal.annotations = dashboard.AnnotationContainer()
        assert isinstance(self._internal.annotations, dashboard.AnnotationContainer)
        annotations_resources = [r1.build() for r1 in annotations]
        self._internal.annotations.list_val = annotations_resources
    
        return self
    
    def annotation(self, annotation: cogbuilder.Builder[dashboard.AnnotationQuery]) -> typing.Self:    
        """
        Contains the list of annotations that are associated with the dashboard.
        Annotations are used to overlay event markers and overlay event tags on graphs.
        Grafana comes with a native annotation store and the ability to add annotation events directly from the graph panel or via the HTTP API.
        See https://grafana.com/docs/grafana/latest/dashboards/build-dashboards/annotate-visualizations/
        """
            
        if self._internal.annotations is None:
            self._internal.annotations = dashboard.AnnotationContainer()
        assert isinstance(self._internal.annotations, dashboard.AnnotationContainer)
        if self._internal.annotations.list_val is None:
            self._internal.annotations.list_val = []
        
        annotation_resource = annotation.build()
        self._internal.annotations.list_val.append(annotation_resource)
    
        return self
    
    def links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        Links with references to other dashboards or external websites.
        """
            
        links_resources = [r1.build() for r1 in links]
        self._internal.links = links_resources
    
        return self
    
    def link(self, link: cogbuilder.Builder[dashboard.DashboardLink]) -> typing.Self:    
        """
        Links with references to other dashboards or external websites.
        """
            
        if self._internal.links is None:
            self._internal.links = []
        
        link_resource = link.build()
        self._internal.links.append(link_resource)
    
        return self
    
    def snapshot(self, snapshot: cogbuilder.Builder[dashboard.Snapshot]) -> typing.Self:    
        """
        Snapshot options. They are present only if the dashboard is a snapshot.
        """
            
        snapshot_resource = snapshot.build()
        self._internal.snapshot = snapshot_resource
    
        return self
    


class TimePicker(cogbuilder.Builder[dashboard.TimePickerConfig]):    
    """
    Time picker configuration
    It defines the default config for the time picker and the refresh picker for the specific dashboard.
    """
    
    _internal: dashboard.TimePickerConfig

    def __init__(self):
        self._internal = dashboard.TimePickerConfig()

    def build(self) -> dashboard.TimePickerConfig:
        """
        Builds the object.
        """
        return self._internal    
    
    def hidden(self, hidden: bool) -> typing.Self:    
        """
        Whether timepicker is visible or not.
        """
            
        self._internal.hidden = hidden
    
        return self
    
    def refresh_intervals(self, refresh_intervals: list[str]) -> typing.Self:    
        """
        Interval options available in the refresh picker dropdown.
        """
            
        self._internal.refresh_intervals = refresh_intervals
    
        return self
    
    def time_options(self, time_options: list[str]) -> typing.Self:    
        """
        Selectable options available in the time picker dropdown. Has no effect on provisioned dashboard.
        """
            
        self._internal.time_options = time_options
    
        return self
    


class Panel(cogbuilder.Builder[dashboard.Panel]):    
    """
    Dashboard panels are the basic visualization building blocks.
    """
    
    _internal: dashboard.Panel

    def __init__(self):
        self._internal = dashboard.Panel()

    def build(self) -> dashboard.Panel:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: str) -> typing.Self:    
        """
        The panel plugin type id. This is used to find the plugin to display the panel.
        """
            
        if not len(type_val) >= 1:
            raise ValueError("len(type_val) must be >= 1")
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: int) -> typing.Self:    
        """
        Unique identifier of the panel. Generated by Grafana when creating a new panel. It must be unique within a dashboard, but not globally.
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def plugin_version(self, plugin_version: str) -> typing.Self:    
        """
        The version of the plugin that is used for this panel. This is used to find the plugin to display the panel and to migrate old panel configs.
        """
            
        self._internal.plugin_version = plugin_version
    
        return self
    
    def tags(self, tags: list[str]) -> typing.Self:    
        """
        Tags for the panel.
        """
            
        self._internal.tags = tags
    
        return self
    
    def targets(self, targets: list[cogbuilder.Builder[cogvariants.Dataquery]]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        targets_resources = [r1.build() for r1 in targets]
        self._internal.targets = targets_resources
    
        return self
    
    def with_target(self, target: cogbuilder.Builder[cogvariants.Dataquery]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        if self._internal.targets is None:
            self._internal.targets = []
        
        target_resource = target.build()
        self._internal.targets.append(target_resource)
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Panel title.
        """
            
        self._internal.title = title
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Panel description.
        """
            
        self._internal.description = description
    
        return self
    
    def transparent(self, transparent: bool) -> typing.Self:    
        """
        Whether to display the panel without a background.
        """
            
        self._internal.transparent = transparent
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource used in all targets.
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def grid_pos(self, grid_pos: dashboard.GridPos) -> typing.Self:    
        """
        Grid position.
        """
            
        self._internal.grid_pos = grid_pos
    
        return self
    
    def height(self, h: int) -> typing.Self:    
        """
        Panel height. The height is the number of rows from the top edge of the panel.
        """
            
        if not h > 0:
            raise ValueError("h must be > 0")
        if self._internal.grid_pos is None:
            self._internal.grid_pos = dashboard.GridPos()
        assert isinstance(self._internal.grid_pos, dashboard.GridPos)
        self._internal.grid_pos.h = h
    
        return self
    
    def span(self, w: int) -> typing.Self:    
        """
        Panel width. The width is the number of columns from the left edge of the panel.
        """
            
        if not w > 0:
            raise ValueError("w must be > 0")
        if not w <= 24:
            raise ValueError("w must be <= 24")
        if self._internal.grid_pos is None:
            self._internal.grid_pos = dashboard.GridPos()
        assert isinstance(self._internal.grid_pos, dashboard.GridPos)
        self._internal.grid_pos.w = w
    
        return self
    
    def links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        Panel links.
        """
            
        links_resources = [r1.build() for r1 in links]
        self._internal.links = links_resources
    
        return self
    
    def repeat(self, repeat: str) -> typing.Self:    
        """
        Name of template variable to repeat for.
        """
            
        self._internal.repeat = repeat
    
        return self
    
    def repeat_direction(self, repeat_direction: typing.Literal["h", "v"]) -> typing.Self:    
        """
        Direction to repeat in if 'repeat' is set.
        `h` for horizontal, `v` for vertical.
        """
            
        self._internal.repeat_direction = repeat_direction
    
        return self
    
    def max_per_row(self, max_per_row: float) -> typing.Self:    
        """
        Option for repeated panels that controls max items per row
        Only relevant for horizontally repeated panels
        """
            
        self._internal.max_per_row = max_per_row
    
        return self
    
    def max_data_points(self, max_data_points: float) -> typing.Self:    
        """
        The maximum number of data points that the panel queries are retrieving.
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def transformations(self, transformations: list[dashboard.DataTransformerConfig]) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        self._internal.transformations = transformations
    
        return self
    
    def with_transformation(self, transformation: dashboard.DataTransformerConfig) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        if self._internal.transformations is None:
            self._internal.transformations = []
        
        self._internal.transformations.append(transformation)
    
        return self
    
    def interval(self, interval: str) -> typing.Self:    
        """
        The min time interval setting defines a lower limit for the $__interval and $__interval_ms variables.
        This value must be formatted as a number followed by a valid time
        identifier like: "40s", "3d", etc.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.interval = interval
    
        return self
    
    def time_from(self, time_from: str) -> typing.Self:    
        """
        Overrides the relative time range for individual panels,
        which causes them to be different than what is selected in
        the dashboard time picker in the top-right corner of the dashboard. You can use this to show metrics from different
        time periods or days on the same dashboard.
        The value is formatted as time operation like: `now-5m` (Last 5 minutes), `now/d` (the day so far),
        `now-5d/d`(Last 5 days), `now/w` (This week so far), `now-2y/y` (Last 2 years).
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_from = time_from
    
        return self
    
    def time_shift(self, time_shift: str) -> typing.Self:    
        """
        Overrides the time range for individual panels by shifting its start and end relative to the time picker.
        For example, you can shift the time range for the panel to be two hours earlier than the dashboard time picker setting `2h`.
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_shift = time_shift
    
        return self
    
    def hide_time_override(self, hide_time_override: bool) -> typing.Self:    
        """
        Controls if the timeFrom or timeShift overrides are shown in the panel header
        """
            
        self._internal.hide_time_override = hide_time_override
    
        return self
    
    def library_panel(self, library_panel: dashboard.LibraryPanelRef) -> typing.Self:    
        """
        Dynamically load the panel
        """
            
        self._internal.library_panel = library_panel
    
        return self
    
    def options(self, options: object) -> typing.Self:    
        """
        It depends on the panel plugin. They are specified by the Options field in panel plugin schemas.
        """
            
        self._internal.options = options
    
        return self
    
    def field_config(self, field_config: dashboard.FieldConfigSource) -> typing.Self:    
        """
        Field options allow you to change how the data is displayed in your visualizations.
        """
            
        self._internal.field_config = field_config
    
        return self
    
    def display_name(self, display_name: str) -> typing.Self:    
        """
        The display value for this field.  This supports template variables blank is auto
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.display_name = display_name
    
        return self
    
    def unit(self, unit: str) -> typing.Self:    
        """
        Unit a field should use. The unit you select is applied to all fields except time.
        You can use the units ID availables in Grafana or a custom unit.
        Available units in Grafana: https://github.com/grafana/grafana/blob/main/packages/grafana-data/src/valueFormats/categories.ts
        As custom unit, you can use the following formats:
        `suffix:<suffix>` for custom unit that should go after value.
        `prefix:<prefix>` for custom unit that should go before value.
        `time:<format>` For custom date time formats type for example `time:YYYY-MM-DD`.
        `si:<base scale><unit characters>` for custom SI units. For example: `si: mF`. This one is a bit more advanced as you can specify both a unit and the source data scale. So if your source data is represented as milli (thousands of) something prefix the unit with that SI scale character.
        `count:<unit>` for a custom count unit.
        `currency:<unit>` for custom a currency unit.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.unit = unit
    
        return self
    
    def decimals(self, decimals: float) -> typing.Self:    
        """
        Specify the number of decimals Grafana includes in the rendered value.
        If you leave this field blank, Grafana automatically truncates the number of decimals based on the value.
        For example 1.1234 will display as 1.12 and 100.456 will display as 100.
        To display all decimals, set the unit to `String`.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.decimals = decimals
    
        return self
    
    def min(self, min_val: float) -> typing.Self:    
        """
        The minimum value used in percentage threshold calculations. Leave blank for auto calculation based on all series and fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.min_val = min_val
    
        return self
    
    def max(self, max_val: float) -> typing.Self:    
        """
        The maximum value used in percentage threshold calculations. Leave blank for auto calculation based on all series and fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.max_val = max_val
    
        return self
    
    def mappings(self, mappings: list[dashboard.ValueMapping]) -> typing.Self:    
        """
        Convert input values into a display string
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.mappings = mappings
    
        return self
    
    def thresholds(self, thresholds: cogbuilder.Builder[dashboard.ThresholdsConfig]) -> typing.Self:    
        """
        Map numeric values to states
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        thresholds_resource = thresholds.build()
        self._internal.field_config.defaults.thresholds = thresholds_resource
    
        return self
    
    def color_scheme(self, color: cogbuilder.Builder[dashboard.FieldColor]) -> typing.Self:    
        """
        Panel color configuration
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        color_resource = color.build()
        self._internal.field_config.defaults.color = color_resource
    
        return self
    
    def data_links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        The behavior when clicking on a result
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        links_resources = [r1.build() for r1 in links]
        self._internal.field_config.defaults.links = links_resources
    
        return self
    
    def no_value(self, no_value: str) -> typing.Self:    
        """
        Alternative to empty string
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.no_value = no_value
    
        return self
    
    def custom(self, custom: object) -> typing.Self:    
        """
        custom is specified by the FieldConfig field
        in panel plugin schemas.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.custom = custom
    
        return self
    
    def defaults(self, defaults: dashboard.FieldConfig) -> typing.Self:    
        """
        Defaults are the options applied to all fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        self._internal.field_config.defaults = defaults
    
        return self
    
    def overrides(self, overrides: list[cogbuilder.Builder[dashboard.DashboardFieldConfigSourceOverrides]]) -> typing.Self:    
        """
        Overrides are the options applied to specific fields overriding the defaults.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        overrides_resources = [r1.build() for r1 in overrides]
        self._internal.field_config.overrides = overrides_resources
    
        return self
    
    def with_override(self, matcher: dashboard.MatcherConfig, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Overrides are the options applied to specific fields overriding the defaults.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=matcher,
            properties=properties,
        ))
    
        return self
    
    def override_by_name(self, name: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for a specific field, referred to by its name.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byName",
            options=name,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_regexp(self, regexp: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for the fields whose name match the given regexp.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byRegexp",
            options=regexp,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_field_type(self, field_type: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for all the fields of the given type.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byType",
            options=field_type,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_query(self, query_ref_id: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byFrameRefID",
            options=query_ref_id,
        ),
            properties=properties,
        ))
    
        return self
    


class DashboardLink(cogbuilder.Builder[dashboard.DashboardLink]):    
    """
    Links with references to other dashboards or external resources
    """
    
    _internal: dashboard.DashboardLink

    def __init__(self, title: str):
        self._internal = dashboard.DashboardLink()        
        self._internal.title = title

    def build(self) -> dashboard.DashboardLink:
        """
        Builds the object.
        """
        return self._internal    
    
    def title(self, title: str) -> typing.Self:    
        """
        Title to display with the link
        """
            
        self._internal.title = title
    
        return self
    
    def type(self, type_val: dashboard.DashboardLinkType) -> typing.Self:    
        """
        Link type. Accepted values are dashboards (to refer to another dashboard) and link (to refer to an external resource)
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def icon(self, icon: str) -> typing.Self:    
        """
        Icon name to be displayed with the link
        """
            
        self._internal.icon = icon
    
        return self
    
    def tooltip(self, tooltip: str) -> typing.Self:    
        """
        Tooltip to display when the user hovers their mouse over it
        """
            
        self._internal.tooltip = tooltip
    
        return self
    
    def url(self, url: str) -> typing.Self:    
        """
        Link URL. Only required/valid if the type is link
        """
            
        self._internal.url = url
    
        return self
    
    def tags(self, tags: list[str]) -> typing.Self:    
        """
        List of tags to limit the linked dashboards. If empty, all dashboards will be displayed. Only valid if the type is dashboards
        """
            
        self._internal.tags = tags
    
        return self
    
    def as_dropdown(self, as_dropdown: bool) -> typing.Self:    
        """
        If true, all dashboards links will be displayed in a dropdown. If false, all dashboards links will be displayed side by side. Only valid if the type is dashboards
        """
            
        self._internal.as_dropdown = as_dropdown
    
        return self
    
    def target_blank(self, target_blank: bool) -> typing.Self:    
        """
        If true, the link will be opened in a new tab
        """
            
        self._internal.target_blank = target_blank
    
        return self
    
    def include_vars(self, include_vars: bool) -> typing.Self:    
        """
        If true, includes current template variables values in the link as query params
        """
            
        self._internal.include_vars = include_vars
    
        return self
    
    def keep_time(self, keep_time: bool) -> typing.Self:    
        """
        If true, includes current time range in the link as query params
        """
            
        self._internal.keep_time = keep_time
    
        return self
    


class ThresholdsConfig(cogbuilder.Builder[dashboard.ThresholdsConfig]):    
    """
    Thresholds configuration for the panel
    """
    
    _internal: dashboard.ThresholdsConfig

    def __init__(self):
        self._internal = dashboard.ThresholdsConfig()

    def build(self) -> dashboard.ThresholdsConfig:
        """
        Builds the object.
        """
        return self._internal    
    
    def mode(self, mode: dashboard.ThresholdsMode) -> typing.Self:    
        """
        Thresholds mode.
        """
            
        self._internal.mode = mode
    
        return self
    
    def steps(self, steps: list[dashboard.Threshold]) -> typing.Self:    
        """
        Must be sorted by 'value', first value is always -Infinity
        """
            
        self._internal.steps = steps
    
        return self
    


class FieldColor(cogbuilder.Builder[dashboard.FieldColor]):    
    """
    Map a field to a color.
    """
    
    _internal: dashboard.FieldColor

    def __init__(self):
        self._internal = dashboard.FieldColor()

    def build(self) -> dashboard.FieldColor:
        """
        Builds the object.
        """
        return self._internal    
    
    def mode(self, mode: dashboard.FieldColorModeId) -> typing.Self:    
        """
        The main color scheme mode.
        """
            
        self._internal.mode = mode
    
        return self
    
    def fixed_color(self, fixed_color: str) -> typing.Self:    
        """
        The fixed color value for fixed or shades color modes.
        """
            
        self._internal.fixed_color = fixed_color
    
        return self
    
    def series_by(self, series_by: dashboard.FieldColorSeriesByMode) -> typing.Self:    
        """
        Some visualizations need to know how to assign a series color from by value color schemes.
        """
            
        self._internal.series_by = series_by
    
        return self
    


class Row(cogbuilder.Builder[dashboard.RowPanel]):    
    """
    Row panel
    """
    
    _internal: dashboard.RowPanel

    def __init__(self, title: str):
        self._internal = dashboard.RowPanel()        
        self._internal.type_val = "row"        
        self._internal.title = title

    def build(self) -> dashboard.RowPanel:
        """
        Builds the object.
        """
        return self._internal    
    
    def collapsed(self, collapsed: bool) -> typing.Self:    
        """
        Whether this row should be collapsed or not.
        Note: panels added directly to a row will be stripped by Grafana unless the row is collapsed
        """
            
        self._internal.collapsed = collapsed
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Row title
        """
            
        self._internal.title = title
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        Name of default datasource for the row
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def grid_pos(self, grid_pos: dashboard.GridPos) -> typing.Self:    
        """
        Row grid position
        """
            
        self._internal.grid_pos = grid_pos
    
        return self
    
    def id(self, id_val: int) -> typing.Self:    
        """
        Unique identifier of the panel. Generated by Grafana when creating a new panel. It must be unique within a dashboard, but not globally.
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def with_panel(self, panel: cogbuilder.Builder[dashboard.Panel]) -> typing.Self:    
        """
        List of panels in the row
        Note: since panels added directly to a row will be stripped by Grafana unless the row is collapsed,
        this option will set the current row as collapsed.
        """
            
        if self._internal.panels is None:
            self._internal.panels = []
        
        panel_resource = panel.build()
        self._internal.panels.append(panel_resource)    
        self._internal.collapsed = True
    
        return self
    
    def repeat(self, repeat: str) -> typing.Self:    
        """
        Name of template variable to repeat for.
        """
            
        self._internal.repeat = repeat
    
        return self
    


class AnnotationQuery(cogbuilder.Builder[dashboard.AnnotationQuery]):    
    """
    TODO docs
    FROM: AnnotationQuery in grafana-data/src/types/annotations.ts
    """
    
    _internal: dashboard.AnnotationQuery

    def __init__(self):
        self._internal = dashboard.AnnotationQuery()

    def build(self) -> dashboard.AnnotationQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of annotation.
        """
            
        self._internal.name = name
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        Datasource where the annotations data is
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def enable(self, enable: bool) -> typing.Self:    
        """
        When enabled the annotation query is issued with every dashboard refresh
        """
            
        self._internal.enable = enable
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        Annotation queries can be toggled on or off at the top of the dashboard.
        When hide is true, the toggle is not shown in the dashboard.
        """
            
        self._internal.hide = hide
    
        return self
    
    def icon_color(self, icon_color: str) -> typing.Self:    
        """
        Color to use for the annotation event markers
        """
            
        self._internal.icon_color = icon_color
    
        return self
    
    def filter(self, filter_val: cogbuilder.Builder[dashboard.AnnotationPanelFilter]) -> typing.Self:    
        """
        Filters to apply when fetching annotations
        """
            
        filter_val_resource = filter_val.build()
        self._internal.filter_val = filter_val_resource
    
        return self
    
    def target(self, target: cogbuilder.Builder[dashboard.AnnotationTarget]) -> typing.Self:    
        """
        TODO.. this should just be a normal query target
        """
            
        target_resource = target.build()
        self._internal.target = target_resource
    
        return self
    
    def type(self, type_val: str) -> typing.Self:    
        """
        TODO -- this should not exist here, it is based on the --grafana-- datasource
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def built_in(self, built_in: float) -> typing.Self:    
        """
        Set to 1 for the standard annotation query all dashboards have by default.
        """
            
        self._internal.built_in = built_in
    
        return self
    
    def expr(self, expr: str) -> typing.Self:    
        self._internal.expr = expr
    
        return self
    


class AnnotationPanelFilter(cogbuilder.Builder[dashboard.AnnotationPanelFilter]):
    _internal: dashboard.AnnotationPanelFilter

    def __init__(self):
        self._internal = dashboard.AnnotationPanelFilter()

    def build(self) -> dashboard.AnnotationPanelFilter:
        """
        Builds the object.
        """
        return self._internal    
    
    def exclude(self, exclude: bool) -> typing.Self:    
        """
        Should the specified panels be included or excluded
        """
            
        self._internal.exclude = exclude
    
        return self
    
    def ids(self, ids: list[int]) -> typing.Self:    
        """
        Panel IDs that should be included or excluded
        """
            
        self._internal.ids = ids
    
        return self
    


class AnnotationTarget(cogbuilder.Builder[dashboard.AnnotationTarget]):    
    """
    TODO: this should be a regular DataQuery that depends on the selected dashboard
    these match the properties of the "grafana" datasouce that is default in most dashboards
    """
    
    _internal: dashboard.AnnotationTarget

    def __init__(self):
        self._internal = dashboard.AnnotationTarget()

    def build(self) -> dashboard.AnnotationTarget:
        """
        Builds the object.
        """
        return self._internal    
    
    def limit(self, limit: int) -> typing.Self:    
        """
        Only required/valid for the grafana datasource...
        but code+tests is already depending on it so hard to change
        """
            
        self._internal.limit = limit
    
        return self
    
    def match_any(self, match_any: bool) -> typing.Self:    
        """
        Only required/valid for the grafana datasource...
        but code+tests is already depending on it so hard to change
        """
            
        self._internal.match_any = match_any
    
        return self
    
    def tags(self, tags: list[str]) -> typing.Self:    
        """
        Only required/valid for the grafana datasource...
        but code+tests is already depending on it so hard to change
        """
            
        self._internal.tags = tags
    
        return self
    
    def type(self, type_val: str) -> typing.Self:    
        """
        Only required/valid for the grafana datasource...
        but code+tests is already depending on it so hard to change
        """
            
        self._internal.type_val = type_val
    
        return self
    


class Snapshot(cogbuilder.Builder[dashboard.Snapshot]):    
    """
    A dashboard snapshot shares an interactive dashboard publicly.
    It is a read-only version of a dashboard, and is not editable.
    It is possible to create a snapshot of a snapshot.
    Grafana strips away all sensitive information from the dashboard.
    Sensitive information stripped: queries (metric, template,annotation) and panel links.
    """
    
    _internal: dashboard.Snapshot

    def __init__(self):
        self._internal = dashboard.Snapshot()

    def build(self) -> dashboard.Snapshot:
        """
        Builds the object.
        """
        return self._internal    
    
    def expires(self, expires: str) -> typing.Self:    
        """
        Time when the snapshot expires, default is never to expire
        """
            
        self._internal.expires = expires
    
        return self
    
    def external(self, external: bool) -> typing.Self:    
        """
        Is the snapshot saved in an external grafana instance
        """
            
        self._internal.external = external
    
        return self
    
    def external_url(self, external_url: str) -> typing.Self:    
        """
        external url, if snapshot was shared in external grafana instance
        """
            
        self._internal.external_url = external_url
    
        return self
    
    def id(self, id_val: int) -> typing.Self:    
        """
        Unique identifier of the snapshot
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def key(self, key: str) -> typing.Self:    
        """
        Optional, defined the unique key of the snapshot, required if external is true
        """
            
        self._internal.key = key
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        """
        Optional, name of the snapshot
        """
            
        self._internal.name = name
    
        return self
    
    def org_id(self, org_id: int) -> typing.Self:    
        """
        org id of the snapshot
        """
            
        self._internal.org_id = org_id
    
        return self
    
    def url(self, url: str) -> typing.Self:    
        """
        url of the snapshot, if snapshot was shared internally
        """
            
        self._internal.url = url
    
        return self
    
    def dashboard(self, dashboard: cogbuilder.Builder[dashboard.Dashboard]) -> typing.Self:    
        dashboard_resource = dashboard.build()
        self._internal.dashboard = dashboard_resource
    
        return self
    


class AnnotationActions(cogbuilder.Builder[dashboard.AnnotationActions]):
    _internal: dashboard.AnnotationActions

    def __init__(self):
        self._internal = dashboard.AnnotationActions()

    def build(self) -> dashboard.AnnotationActions:
        """
        Builds the object.
        """
        return self._internal    
    
    def can_add(self, can_add: bool) -> typing.Self:    
        self._internal.can_add = can_add
    
        return self
    
    def can_delete(self, can_delete: bool) -> typing.Self:    
        self._internal.can_delete = can_delete
    
        return self
    
    def can_edit(self, can_edit: bool) -> typing.Self:    
        self._internal.can_edit = can_edit
    
        return self
    


class AnnotationPermission(cogbuilder.Builder[dashboard.AnnotationPermission]):
    _internal: dashboard.AnnotationPermission

    def __init__(self):
        self._internal = dashboard.AnnotationPermission()

    def build(self) -> dashboard.AnnotationPermission:
        """
        Builds the object.
        """
        return self._internal    
    
    def dashboard_permissions(self, dashboard: cogbuilder.Builder[dashboard.AnnotationActions]) -> typing.Self:    
        dashboard_resource = dashboard.build()
        self._internal.dashboard = dashboard_resource
    
        return self
    
    def organization_permissions(self, organization: cogbuilder.Builder[dashboard.AnnotationActions]) -> typing.Self:    
        organization_resource = organization.build()
        self._internal.organization = organization_resource
    
        return self
    


class DashboardMeta(cogbuilder.Builder[dashboard.DashboardMeta]):
    _internal: dashboard.DashboardMeta

    def __init__(self):
        self._internal = dashboard.DashboardMeta()

    def build(self) -> dashboard.DashboardMeta:
        """
        Builds the object.
        """
        return self._internal    
    
    def annotations_permissions(self, annotations_permissions: cogbuilder.Builder[dashboard.AnnotationPermission]) -> typing.Self:    
        annotations_permissions_resource = annotations_permissions.build()
        self._internal.annotations_permissions = annotations_permissions_resource
    
        return self
    
    def can_admin(self, can_admin: bool) -> typing.Self:    
        self._internal.can_admin = can_admin
    
        return self
    
    def can_delete(self, can_delete: bool) -> typing.Self:    
        self._internal.can_delete = can_delete
    
        return self
    
    def can_edit(self, can_edit: bool) -> typing.Self:    
        self._internal.can_edit = can_edit
    
        return self
    
    def can_save(self, can_save: bool) -> typing.Self:    
        self._internal.can_save = can_save
    
        return self
    
    def can_star(self, can_star: bool) -> typing.Self:    
        self._internal.can_star = can_star
    
        return self
    
    def created(self, created: str) -> typing.Self:    
        self._internal.created = created
    
        return self
    
    def created_by(self, created_by: str) -> typing.Self:    
        self._internal.created_by = created_by
    
        return self
    
    def expires(self, expires: str) -> typing.Self:    
        self._internal.expires = expires
    
        return self
    
    def folder_id(self, folder_id: int) -> typing.Self:    
        """
        Deprecated: use FolderUID instead
        """
            
        self._internal.folder_id = folder_id
    
        return self
    
    def folder_title(self, folder_title: str) -> typing.Self:    
        self._internal.folder_title = folder_title
    
        return self
    
    def folder_uid(self, folder_uid: str) -> typing.Self:    
        self._internal.folder_uid = folder_uid
    
        return self
    
    def folder_url(self, folder_url: str) -> typing.Self:    
        self._internal.folder_url = folder_url
    
        return self
    
    def has_acl(self, has_acl: bool) -> typing.Self:    
        self._internal.has_acl = has_acl
    
        return self
    
    def is_folder(self, is_folder: bool) -> typing.Self:    
        self._internal.is_folder = is_folder
    
        return self
    
    def is_snapshot(self, is_snapshot: bool) -> typing.Self:    
        self._internal.is_snapshot = is_snapshot
    
        return self
    
    def is_starred(self, is_starred: bool) -> typing.Self:    
        self._internal.is_starred = is_starred
    
        return self
    
    def provisioned(self, provisioned: bool) -> typing.Self:    
        self._internal.provisioned = provisioned
    
        return self
    
    def provisioned_external_id(self, provisioned_external_id: str) -> typing.Self:    
        self._internal.provisioned_external_id = provisioned_external_id
    
        return self
    
    def public_dashboard_enabled(self, public_dashboard_enabled: bool) -> typing.Self:    
        self._internal.public_dashboard_enabled = public_dashboard_enabled
    
        return self
    
    def public_dashboard_uid(self, public_dashboard_uid: str) -> typing.Self:    
        self._internal.public_dashboard_uid = public_dashboard_uid
    
        return self
    
    def slug(self, slug: str) -> typing.Self:    
        self._internal.slug = slug
    
        return self
    
    def type(self, type_val: str) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def updated(self, updated: str) -> typing.Self:    
        self._internal.updated = updated
    
        return self
    
    def updated_by(self, updated_by: str) -> typing.Self:    
        self._internal.updated_by = updated_by
    
        return self
    
    def url(self, url: str) -> typing.Self:    
        self._internal.url = url
    
        return self
    
    def version(self, version: int) -> typing.Self:    
        self._internal.version = version
    
        return self
    


class DashboardDashboardTemplating(cogbuilder.Builder[dashboard.DashboardDashboardTemplating]):
    _internal: dashboard.DashboardDashboardTemplating

    def __init__(self):
        self._internal = dashboard.DashboardDashboardTemplating()

    def build(self) -> dashboard.DashboardDashboardTemplating:
        """
        Builds the object.
        """
        return self._internal    
    
    def list(self, list_val: list[cogbuilder.Builder[dashboard.VariableModel]]) -> typing.Self:    
        """
        List of configured template variables with their saved values along with some other metadata
        """
            
        list_val_resources = [r1.build() for r1 in list_val]
        self._internal.list_val = list_val_resources
    
        return self
    


class DashboardFieldConfigSourceOverrides(cogbuilder.Builder[dashboard.DashboardFieldConfigSourceOverrides]):
    _internal: dashboard.DashboardFieldConfigSourceOverrides

    def __init__(self):
        self._internal = dashboard.DashboardFieldConfigSourceOverrides()

    def build(self) -> dashboard.DashboardFieldConfigSourceOverrides:
        """
        Builds the object.
        """
        return self._internal    
    
    def matcher(self, matcher: dashboard.MatcherConfig) -> typing.Self:    
        self._internal.matcher = matcher
    
        return self
    
    def properties(self, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        self._internal.properties = properties
    
        return self
    


class DashboardRangeMapOptions(cogbuilder.Builder[dashboard.DashboardRangeMapOptions]):
    _internal: dashboard.DashboardRangeMapOptions

    def __init__(self):
        self._internal = dashboard.DashboardRangeMapOptions()

    def build(self) -> dashboard.DashboardRangeMapOptions:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: float) -> typing.Self:    
        """
        Min value of the range. It can be null which means -Infinity
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: float) -> typing.Self:    
        """
        Max value of the range. It can be null which means +Infinity
        """
            
        self._internal.to = to
    
        return self
    
    def result(self, result: dashboard.ValueMappingResult) -> typing.Self:    
        """
        Config to apply when the value is within the range
        """
            
        self._internal.result = result
    
        return self
    


class DashboardRegexMapOptions(cogbuilder.Builder[dashboard.DashboardRegexMapOptions]):
    _internal: dashboard.DashboardRegexMapOptions

    def __init__(self):
        self._internal = dashboard.DashboardRegexMapOptions()

    def build(self) -> dashboard.DashboardRegexMapOptions:
        """
        Builds the object.
        """
        return self._internal    
    
    def pattern(self, pattern: str) -> typing.Self:    
        """
        Regular expression to match against
        """
            
        self._internal.pattern = pattern
    
        return self
    
    def result(self, result: dashboard.ValueMappingResult) -> typing.Self:    
        """
        Config to apply when the value matches the regex
        """
            
        self._internal.result = result
    
        return self
    


class DashboardSpecialValueMapOptions(cogbuilder.Builder[dashboard.DashboardSpecialValueMapOptions]):
    _internal: dashboard.DashboardSpecialValueMapOptions

    def __init__(self):
        self._internal = dashboard.DashboardSpecialValueMapOptions()

    def build(self) -> dashboard.DashboardSpecialValueMapOptions:
        """
        Builds the object.
        """
        return self._internal    
    
    def match(self, match: dashboard.SpecialValueMatch) -> typing.Self:    
        """
        Special value to match against
        """
            
        self._internal.match = match
    
        return self
    
    def result(self, result: dashboard.ValueMappingResult) -> typing.Self:    
        """
        Config to apply when the value matches the special value
        """
            
        self._internal.result = result
    
        return self
    


class QueryVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.QUERY

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def hide(self, hide: dashboard.VariableHide) -> typing.Self:    
        """
        Visibility configuration for the variable
        """
            
        self._internal.hide = hide
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def query(self, query: typing.Union[str, dict[str, object]]) -> typing.Self:    
        """
        Query used to fetch values for a variable
        """
            
        self._internal.query = query
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        Data source used to fetch values for a variable. It can be defined but `null`.
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def current(self, current: dashboard.VariableOption) -> typing.Self:    
        """
        Shows current selected variable text/value on the dashboard
        """
            
        self._internal.current = current
    
        return self
    
    def multi(self, multi: bool) -> typing.Self:    
        """
        Whether multiple values can be selected or not from variable value list
        """
            
        self._internal.multi = multi
    
        return self
    
    def options(self, options: list[dashboard.VariableOption]) -> typing.Self:    
        """
        Options that can be selected for a variable.
        """
            
        self._internal.options = options
    
        return self
    
    def refresh(self, refresh: dashboard.VariableRefresh) -> typing.Self:    
        """
        Options to config when to refresh a variable
        """
            
        self._internal.refresh = refresh
    
        return self
    
    def sort(self, sort: dashboard.VariableSort) -> typing.Self:    
        """
        Options sort order
        """
            
        self._internal.sort = sort
    
        return self
    
    def include_all(self, include_all: bool) -> typing.Self:    
        """
        Whether all value option is available or not
        """
            
        self._internal.include_all = include_all
    
        return self
    
    def all_value(self, all_value: str) -> typing.Self:    
        """
        Custom all value
        """
            
        self._internal.all_value = all_value
    
        return self
    
    def regex(self, regex: str) -> typing.Self:    
        """
        Optional field, if you want to extract part of a series name or metric node segment.
        Named capture groups can be used to separate the display text and value.
        """
            
        self._internal.regex = regex
    
        return self
    


class AdHocVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.ADHOC

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def hide(self, hide: dashboard.VariableHide) -> typing.Self:    
        """
        Visibility configuration for the variable
        """
            
        self._internal.hide = hide
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        Data source used to fetch values for a variable. It can be defined but `null`.
        """
            
        self._internal.datasource = datasource
    
        return self
    


class ConstantVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.CONSTANT        
        self._internal.hide = dashboard.VariableHide.HIDE_VARIABLE

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def value(self, query: typing.Union[str, dict[str, object]]) -> typing.Self:    
        """
        Query used to fetch values for a variable
        """
            
        self._internal.query = query
    
        return self
    


class DatasourceVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.DATASOURCE

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def hide(self, hide: dashboard.VariableHide) -> typing.Self:    
        """
        Visibility configuration for the variable
        """
            
        self._internal.hide = hide
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def type(self, query: typing.Union[str, dict[str, object]]) -> typing.Self:    
        """
        Query used to fetch values for a variable
        """
            
        self._internal.query = query
    
        return self
    
    def current(self, current: dashboard.VariableOption) -> typing.Self:    
        """
        Shows current selected variable text/value on the dashboard
        """
            
        self._internal.current = current
    
        return self
    
    def multi(self, multi: bool) -> typing.Self:    
        """
        Whether multiple values can be selected or not from variable value list
        """
            
        self._internal.multi = multi
    
        return self
    
    def include_all(self, include_all: bool) -> typing.Self:    
        """
        Whether all value option is available or not
        """
            
        self._internal.include_all = include_all
    
        return self
    
    def all_value(self, all_value: str) -> typing.Self:    
        """
        Custom all value
        """
            
        self._internal.all_value = all_value
    
        return self
    
    def regex(self, regex: str) -> typing.Self:    
        """
        Optional field, if you want to extract part of a series name or metric node segment.
        Named capture groups can be used to separate the display text and value.
        """
            
        self._internal.regex = regex
    
        return self
    


class IntervalVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.INTERVAL

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def hide(self, hide: dashboard.VariableHide) -> typing.Self:    
        """
        Visibility configuration for the variable
        """
            
        self._internal.hide = hide
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def values(self, query: typing.Union[str, dict[str, object]]) -> typing.Self:    
        """
        Query used to fetch values for a variable
        """
            
        self._internal.query = query
    
        return self
    
    def current(self, current: dashboard.VariableOption) -> typing.Self:    
        """
        Shows current selected variable text/value on the dashboard
        """
            
        self._internal.current = current
    
        return self
    
    def options(self, options: list[dashboard.VariableOption]) -> typing.Self:    
        """
        Options that can be selected for a variable.
        """
            
        self._internal.options = options
    
        return self
    
    def auto(self, auto: bool) -> typing.Self:    
        """
        Dynamically calculates interval by dividing time range by the count specified.
        """
            
        self._internal.auto = auto
    
        return self
    
    def min_interval(self, auto_min: str) -> typing.Self:    
        """
        The minimum threshold below which the step count intervals will not divide the time.
        """
            
        self._internal.auto_min = auto_min
    
        return self
    
    def step_count(self, auto_count: int) -> typing.Self:    
        """
        How many times the current time range should be divided to calculate the value, similar to the Max data points query option.
        For example, if the current visible time range is 30 minutes, then the auto interval groups the data into 30 one-minute increments.
        """
            
        if not auto_count > 0:
            raise ValueError("auto_count must be > 0")
        self._internal.auto_count = auto_count
    
        return self
    


class TextBoxVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.TEXTBOX

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def hide(self, hide: dashboard.VariableHide) -> typing.Self:    
        """
        Visibility configuration for the variable
        """
            
        self._internal.hide = hide
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def default_value(self, query: typing.Union[str, dict[str, object]]) -> typing.Self:    
        """
        Query used to fetch values for a variable
        """
            
        self._internal.query = query
    
        return self
    
    def current(self, current: dashboard.VariableOption) -> typing.Self:    
        """
        Shows current selected variable text/value on the dashboard
        """
            
        self._internal.current = current
    
        return self
    
    def options(self, options: list[dashboard.VariableOption]) -> typing.Self:    
        """
        Options that can be selected for a variable.
        """
            
        self._internal.options = options
    
        return self
    


class CustomVariable(cogbuilder.Builder[dashboard.VariableModel]):    
    """
    A variable is a placeholder for a value. You can use variables in metric queries and in panel titles.
    """
    
    _internal: dashboard.VariableModel

    def __init__(self, name: str):
        self._internal = dashboard.VariableModel()        
        self._internal.name = name        
        self._internal.type_val = dashboard.VariableType.CUSTOM

    def build(self) -> dashboard.VariableModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        """
        Name of variable
        """
            
        self._internal.name = name
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        """
        Optional display name
        """
            
        self._internal.label = label
    
        return self
    
    def hide(self, hide: dashboard.VariableHide) -> typing.Self:    
        """
        Visibility configuration for the variable
        """
            
        self._internal.hide = hide
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of variable. It can be defined but `null`.
        """
            
        self._internal.description = description
    
        return self
    
    def values(self, query: typing.Union[str, dict[str, object]]) -> typing.Self:    
        """
        Query used to fetch values for a variable
        """
            
        self._internal.query = query
    
        return self
    
    def current(self, current: dashboard.VariableOption) -> typing.Self:    
        """
        Shows current selected variable text/value on the dashboard
        """
            
        self._internal.current = current
    
        return self
    
    def multi(self, multi: bool) -> typing.Self:    
        """
        Whether multiple values can be selected or not from variable value list
        """
            
        self._internal.multi = multi
    
        return self
    
    def options(self, options: list[dashboard.VariableOption]) -> typing.Self:    
        """
        Options that can be selected for a variable.
        """
            
        self._internal.options = options
    
        return self
    
    def include_all(self, include_all: bool) -> typing.Self:    
        """
        Whether all value option is available or not
        """
            
        self._internal.include_all = include_all
    
        return self
    
    def all_value(self, all_value: str) -> typing.Self:    
        """
        Custom all value
        """
            
        self._internal.all_value = all_value
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
import enum
from ..cog import runtime as cogruntime


class Options:
    view: 'MapViewConfig'
    controls: 'ControlsOptions'
    basemap: common.MapLayerOptions
    layers: list[common.MapLayerOptions]
    tooltip: 'TooltipOptions'

    def __init__(self, view: typing.Optional['MapViewConfig'] = None, controls: typing.Optional['ControlsOptions'] = None, basemap: typing.Optional[common.MapLayerOptions] = None, layers: typing.Optional[list[common.MapLayerOptions]] = None, tooltip: typing.Optional['TooltipOptions'] = None):
        self.view = view if view is not None else MapViewConfig()
        self.controls = controls if controls is not None else ControlsOptions()
        self.basemap = basemap if basemap is not None else common.MapLayerOptions()
        self.layers = layers if layers is not None else []
        self.tooltip = tooltip if tooltip is not None else TooltipOptions()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "view": self.view,
            "controls": self.controls,
            "basemap": self.basemap,
            "layers": self.layers,
            "tooltip": self.tooltip,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "view" in data:
            args["view"] = MapViewConfig.from_json(data["view"])
        if "controls" in data:
            args["controls"] = ControlsOptions.from_json(data["controls"])
        if "basemap" in data:
            args["basemap"] = common.MapLayerOptions.from_json(data["basemap"])
        if "layers" in data:
            args["layers"] = [common.MapLayerOptions.from_json(item) for item in data["layers"]]
        if "tooltip" in data:
            args["tooltip"] = TooltipOptions.from_json(data["tooltip"])        

        return cls(**args)


class MapViewConfig:
    id_val: str
    lat: typing.Optional[int]
    lon: typing.Optional[int]
    zoom: typing.Optional[int]
    min_zoom: typing.Optional[int]
    max_zoom: typing.Optional[int]
    padding: typing.Optional[int]
    all_layers: typing.Optional[bool]
    last_only: typing.Optional[bool]
    layer: typing.Optional[str]
    shared: typing.Optional[bool]

    def __init__(self, id_val: str = "zero", lat: typing.Optional[int] = 0, lon: typing.Optional[int] = 0, zoom: typing.Optional[int] = 1, min_zoom: typing.Optional[int] = None, max_zoom: typing.Optional[int] = None, padding: typing.Optional[int] = None, all_layers: typing.Optional[bool] = True, last_only: typing.Optional[bool] = None, layer: typing.Optional[str] = None, shared: typing.Optional[bool] = None):
        self.id_val = id_val
        self.lat = lat
        self.lon = lon
        self.zoom = zoom
        self.min_zoom = min_zoom
        self.max_zoom = max_zoom
        self.padding = padding
        self.all_layers = all_layers
        self.last_only = last_only
        self.layer = layer
        self.shared = shared

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
        }
        if self.lat is not None:
            payload["lat"] = self.lat
        if self.lon is not None:
            payload["lon"] = self.lon
        if self.zoom is not None:
            payload["zoom"] = self.zoom
        if self.min_zoom is not None:
            payload["minZoom"] = self.min_zoom
        if self.max_zoom is not None:
            payload["maxZoom"] = self.max_zoom
        if self.padding is not None:
            payload["padding"] = self.padding
        if self.all_layers is not None:
            payload["allLayers"] = self.all_layers
        if self.last_only is not None:
            payload["lastOnly"] = self.last_only
        if self.layer is not None:
            payload["layer"] = self.layer
        if self.shared is not None:
            payload["shared"] = self.shared
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "lat" in data:
            args["lat"] = data["lat"]
        if "lon" in data:
            args["lon"] = data["lon"]
        if "zoom" in data:
            args["zoom"] = data["zoom"]
        if "minZoom" in data:
            args["min_zoom"] = data["minZoom"]
        if "maxZoom" in data:
            args["max_zoom"] = data["maxZoom"]
        if "padding" in data:
            args["padding"] = data["padding"]
        if "allLayers" in data:
            args["all_layers"] = data["allLayers"]
        if "lastOnly" in data:
            args["last_only"] = data["lastOnly"]
        if "layer" in data:
            args["layer"] = data["layer"]
        if "shared" in data:
            args["shared"] = data["shared"]        

        return cls(**args)


class ControlsOptions:
    # Zoom (upper left)
    show_zoom: typing.Optional[bool]
    # let the mouse wheel zoom
    mouse_wheel_zoom: typing.Optional[bool]
    # Lower right
    show_attribution: typing.Optional[bool]
    # Scale options
    show_scale: typing.Optional[bool]
    # Show debug
    show_debug: typing.Optional[bool]
    # Show measure
    show_measure: typing.Optional[bool]

    def __init__(self, show_zoom: typing.Optional[bool] = None, mouse_wheel_zoom: typing.Optional[bool] = None, show_attribution: typing.Optional[bool] = None, show_scale: typing.Optional[bool] = None, show_debug: typing.Optional[bool] = None, show_measure: typing.Optional[bool] = None):
        self.show_zoom = show_zoom
        self.mouse_wheel_zoom = mouse_wheel_zoom
        self.show_attribution = show_attribution
        self.show_scale = show_scale
        self.show_debug = show_debug
        self.show_measure = show_measure

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.show_zoom is not None:
            payload["showZoom"] = self.show_zoom
        if self.mouse_wheel_zoom is not None:
            payload["mouseWheelZoom"] = self.mouse_wheel_zoom
        if self.show_attribution is not None:
            payload["showAttribution"] = self.show_attribution
        if self.show_scale is not None:
            payload["showScale"] = self.show_scale
        if self.show_debug is not None:
            payload["showDebug"] = self.show_debug
        if self.show_measure is not None:
            payload["showMeasure"] = self.show_measure
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "showZoom" in data:
            args["show_zoom"] = data["showZoom"]
        if "mouseWheelZoom" in data:
            args["mouse_wheel_zoom"] = data["mouseWheelZoom"]
        if "showAttribution" in data:
            args["show_attribution"] = data["showAttribution"]
        if "showScale" in data:
            args["show_scale"] = data["showScale"]
        if "showDebug" in data:
            args["show_debug"] = data["showDebug"]
        if "showMeasure" in data:
            args["show_measure"] = data["showMeasure"]        

        return cls(**args)


class TooltipOptions:
    mode: 'TooltipMode'

    def __init__(self, mode: typing.Optional['TooltipMode'] = None):
        self.mode = mode if mode is not None else TooltipMode.NONE

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "mode": self.mode,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]        

        return cls(**args)


class TooltipMode(enum.StrEnum):
    NONE = "none"
    DETAILS = "details"


class MapCenterID(enum.StrEnum):
    ZERO = "zero"
    COORDS = "coords"
    FIT = "fit"





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="geomap",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
import typing
from ..models import common
from ..cog import runtime as cogruntime


class VizDisplayMode(enum.StrEnum):
    CANDLES_VOLUME = "candles+volume"
    CANDLES = "candles"
    VOLUME = "volume"


class CandleStyle(enum.StrEnum):
    CANDLES = "candles"
    OHLC_BARS = "ohlcbars"


class ColorStrategy(enum.StrEnum):
    OPEN_CLOSE = "open-close"
    CLOSE_CLOSE = "close-close"


class CandlestickFieldMap:
    # Corresponds to the starting value of the given period
    open_val: typing.Optional[str]
    # Corresponds to the highest value of the given period
    high: typing.Optional[str]
    # Corresponds to the lowest value of the given period
    low: typing.Optional[str]
    # Corresponds to the final (end) value of the given period
    close: typing.Optional[str]
    # Corresponds to the sample count in the given period. (e.g. number of trades)
    volume: typing.Optional[str]

    def __init__(self, open_val: typing.Optional[str] = None, high: typing.Optional[str] = None, low: typing.Optional[str] = None, close: typing.Optional[str] = None, volume: typing.Optional[str] = None):
        self.open_val = open_val
        self.high = high
        self.low = low
        self.close = close
        self.volume = volume

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.open_val is not None:
            payload["open"] = self.open_val
        if self.high is not None:
            payload["high"] = self.high
        if self.low is not None:
            payload["low"] = self.low
        if self.close is not None:
            payload["close"] = self.close
        if self.volume is not None:
            payload["volume"] = self.volume
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "open" in data:
            args["open_val"] = data["open"]
        if "high" in data:
            args["high"] = data["high"]
        if "low" in data:
            args["low"] = data["low"]
        if "close" in data:
            args["close"] = data["close"]
        if "volume" in data:
            args["volume"] = data["volume"]        

        return cls(**args)


class CandlestickColors:
    up: str
    down: str
    flat: str

    def __init__(self, up: str = "green", down: str = "red", flat: str = "gray"):
        self.up = up
        self.down = down
        self.flat = flat

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "up": self.up,
            "down": self.down,
            "flat": self.flat,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "up" in data:
            args["up"] = data["up"]
        if "down" in data:
            args["down"] = data["down"]
        if "flat" in data:
            args["flat"] = data["flat"]        

        return cls(**args)


class Options:
    # Sets which dimensions are used for the visualization
    mode: 'VizDisplayMode'
    # Sets the style of the candlesticks
    candle_style: 'CandleStyle'
    # Sets the color strategy for the candlesticks
    color_strategy: 'ColorStrategy'
    # Map fields to appropriate dimension
    fields: 'CandlestickFieldMap'
    # Set which colors are used when the price movement is up or down
    colors: 'CandlestickColors'
    legend: common.VizLegendOptions
    # When enabled, all fields will be sent to the graph
    include_all_fields: typing.Optional[bool]

    def __init__(self, mode: typing.Optional['VizDisplayMode'] = None, candle_style: typing.Optional['CandleStyle'] = None, color_strategy: typing.Optional['ColorStrategy'] = None, fields: typing.Optional['CandlestickFieldMap'] = None, colors: typing.Optional['CandlestickColors'] = None, legend: typing.Optional[common.VizLegendOptions] = None, include_all_fields: typing.Optional[bool] = False):
        self.mode = mode if mode is not None else VizDisplayMode.CANDLES_VOLUME
        self.candle_style = candle_style if candle_style is not None else CandleStyle.CANDLES
        self.color_strategy = color_strategy if color_strategy is not None else ColorStrategy.OPEN_CLOSE
        self.fields = fields if fields is not None else CandlestickFieldMap()
        self.colors = colors if colors is not None else CandlestickColors(down="red", flat="gray", up="green")
        self.legend = legend if legend is not None else common.VizLegendOptions()
        self.include_all_fields = include_all_fields

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "mode": self.mode,
            "candleStyle": self.candle_style,
            "colorStrategy": self.color_strategy,
            "fields": self.fields,
            "colors": self.colors,
            "legend": self.legend,
        }
        if self.include_all_fields is not None:
            payload["includeAllFields"] = self.include_all_fields
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]
        if "candleStyle" in data:
            args["candle_style"] = data["candleStyle"]
        if "colorStrategy" in data:
            args["color_strategy"] = data["colorStrategy"]
        if "fields" in data:
            args["fields"] = CandlestickFieldMap.from_json(data["fields"])
        if "colors" in data:
            args["colors"] = CandlestickColors.from_json(data["colors"])
        if "legend" in data:
            args["legend"] = common.VizLegendOptions.from_json(data["legend"])
        if "includeAllFields" in data:
            args["include_all_fields"] = data["includeAllFields"]        

        return cls(**args)


FieldConfig: typing.TypeAlias = common.GraphFieldConfig





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="candlestick",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


class Team:
    # Name of the team.
    name: str
    # Email of the team.
    email: typing.Optional[str]

    def __init__(self, name: str = "", email: typing.Optional[str] = None):
        self.name = name
        self.email = email

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "name": self.name,
        }
        if self.email is not None:
            payload["email"] = self.email
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "email" in data:
            args["email"] = data["email"]        

        return cls(**args)



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
from ..cog import variants as cogvariants
import typing
from ..models import dashboard
from ..cog import runtime as cogruntime


class QueryEditorMode(enum.StrEnum):
    CODE = "code"
    BUILDER = "builder"


class LokiQueryType(enum.StrEnum):
    RANGE = "range"
    INSTANT = "instant"
    STREAM = "stream"


class SupportingQueryType(enum.StrEnum):
    LOGS_VOLUME = "logsVolume"
    LOGS_SAMPLE = "logsSample"
    DATA_SAMPLE = "dataSample"


class LokiQueryDirection(enum.StrEnum):
    FORWARD = "forward"
    BACKWARD = "backward"


class Dataquery(cogvariants.Dataquery):
    # The LogQL query.
    expr: str
    # Used to override the name of the series.
    legend_format: typing.Optional[str]
    # Used to limit the number of log rows returned.
    max_lines: typing.Optional[int]
    # @deprecated, now use step.
    resolution: typing.Optional[int]
    editor_mode: typing.Optional['QueryEditorMode']
    # @deprecated, now use queryType.
    range_val: typing.Optional[bool]
    # @deprecated, now use queryType.
    instant: typing.Optional[bool]
    # Used to set step value for range queries.
    step: typing.Optional[str]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]

    def __init__(self, expr: str = "", legend_format: typing.Optional[str] = None, max_lines: typing.Optional[int] = None, resolution: typing.Optional[int] = None, editor_mode: typing.Optional['QueryEditorMode'] = None, range_val: typing.Optional[bool] = None, instant: typing.Optional[bool] = None, step: typing.Optional[str] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None):
        self.expr = expr
        self.legend_format = legend_format
        self.max_lines = max_lines
        self.resolution = resolution
        self.editor_mode = editor_mode
        self.range_val = range_val
        self.instant = instant
        self.step = step
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "expr": self.expr,
            "refId": self.ref_id,
        }
        if self.legend_format is not None:
            payload["legendFormat"] = self.legend_format
        if self.max_lines is not None:
            payload["maxLines"] = self.max_lines
        if self.resolution is not None:
            payload["resolution"] = self.resolution
        if self.editor_mode is not None:
            payload["editorMode"] = self.editor_mode
        if self.range_val is not None:
            payload["range"] = self.range_val
        if self.instant is not None:
            payload["instant"] = self.instant
        if self.step is not None:
            payload["step"] = self.step
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "expr" in data:
            args["expr"] = data["expr"]
        if "legendFormat" in data:
            args["legend_format"] = data["legendFormat"]
        if "maxLines" in data:
            args["max_lines"] = data["maxLines"]
        if "resolution" in data:
            args["resolution"] = data["resolution"]
        if "editorMode" in data:
            args["editor_mode"] = data["editorMode"]
        if "range" in data:
            args["range_val"] = data["range"]
        if "instant" in data:
            args["instant"] = data["instant"]
        if "step" in data:
            args["step"] = data["step"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="loki",
        from_json_hook=Dataquery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import variants as cogvariants
from ..models import dashboard
import enum
from ..cog import runtime as cogruntime


class MetricStat:
    # AWS region to query for the metric
    region: str
    # A namespace is a container for CloudWatch metrics. Metrics in different namespaces are isolated from each other, so that metrics from different applications are not mistakenly aggregated into the same statistics. For example, Amazon EC2 uses the AWS/EC2 namespace.
    namespace: str
    # Name of the metric
    metric_name: typing.Optional[str]
    # The dimensions of the metric
    dimensions: typing.Optional['Dimensions']
    # Only show metrics that exactly match all defined dimension names.
    match_exact: typing.Optional[bool]
    # The length of time associated with a specific Amazon CloudWatch statistic. Can be specified by a number of seconds, 'auto', or as a duration string e.g. '15m' being 15 minutes
    period: typing.Optional[str]
    # The ID of the AWS account to query for the metric, specifying `all` will query all accounts that the monitoring account is permitted to query.
    account_id: typing.Optional[str]
    # Metric data aggregations over specified periods of time. For detailed definitions of the statistics supported by CloudWatch, see https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/Statistics-definitions.html.
    statistic: typing.Optional[str]
    # @deprecated use statistic
    statistics: typing.Optional[list[str]]

    def __init__(self, region: str = "", namespace: str = "", metric_name: typing.Optional[str] = None, dimensions: typing.Optional['Dimensions'] = None, match_exact: typing.Optional[bool] = None, period: typing.Optional[str] = None, account_id: typing.Optional[str] = None, statistic: typing.Optional[str] = None, statistics: typing.Optional[list[str]] = None):
        self.region = region
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.match_exact = match_exact
        self.period = period
        self.account_id = account_id
        self.statistic = statistic
        self.statistics = statistics

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "region": self.region,
            "namespace": self.namespace,
        }
        if self.metric_name is not None:
            payload["metricName"] = self.metric_name
        if self.dimensions is not None:
            payload["dimensions"] = self.dimensions
        if self.match_exact is not None:
            payload["matchExact"] = self.match_exact
        if self.period is not None:
            payload["period"] = self.period
        if self.account_id is not None:
            payload["accountId"] = self.account_id
        if self.statistic is not None:
            payload["statistic"] = self.statistic
        if self.statistics is not None:
            payload["statistics"] = self.statistics
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "region" in data:
            args["region"] = data["region"]
        if "namespace" in data:
            args["namespace"] = data["namespace"]
        if "metricName" in data:
            args["metric_name"] = data["metricName"]
        if "dimensions" in data:
            args["dimensions"] = {key: data["dimensions"][key] for key in data["dimensions"].keys()}
        if "matchExact" in data:
            args["match_exact"] = data["matchExact"]
        if "period" in data:
            args["period"] = data["period"]
        if "accountId" in data:
            args["account_id"] = data["accountId"]
        if "statistic" in data:
            args["statistic"] = data["statistic"]
        if "statistics" in data:
            args["statistics"] = data["statistics"]        

        return cls(**args)


# A name/value pair that is part of the identity of a metric. For example, you can get statistics for a specific EC2 instance by specifying the InstanceId dimension when you search for metrics.
Dimensions: typing.TypeAlias = dict[str, typing.Union[str, list[str]]]


class CloudWatchMetricsQuery(cogvariants.Dataquery):
    """
    Shape of a CloudWatch Metrics query
    """

    # Whether a query is a Metrics, Logs, or Annotations query
    query_mode: 'CloudWatchQueryMode'
    # Whether to use a metric search or metric query. Metric query is referred to as "Metrics Insights" in the AWS console.
    metric_query_type: typing.Optional['MetricQueryType']
    # Whether to use the query builder or code editor to create the query
    metric_editor_mode: typing.Optional['MetricEditorMode']
    # ID can be used to reference other queries in math expressions. The ID can include numbers, letters, and underscore, and must start with a lowercase letter.
    id_val: str
    # Deprecated: use label
    # @deprecated use label
    alias: typing.Optional[str]
    # Change the time series legend names using dynamic labels. See https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/graph-dynamic-labels.html for more details.
    label: typing.Optional[str]
    # Math expression query
    expression: typing.Optional[str]
    # When the metric query type is `metricQueryType` is set to `Query`, this field is used to specify the query string.
    sql_expression: typing.Optional[str]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # AWS region to query for the metric
    region: str
    # A namespace is a container for CloudWatch metrics. Metrics in different namespaces are isolated from each other, so that metrics from different applications are not mistakenly aggregated into the same statistics. For example, Amazon EC2 uses the AWS/EC2 namespace.
    namespace: str
    # Name of the metric
    metric_name: typing.Optional[str]
    # The dimensions of the metric
    dimensions: typing.Optional['Dimensions']
    # Only show metrics that exactly match all defined dimension names.
    match_exact: typing.Optional[bool]
    # The length of time associated with a specific Amazon CloudWatch statistic. Can be specified by a number of seconds, 'auto', or as a duration string e.g. '15m' being 15 minutes
    period: typing.Optional[str]
    # The ID of the AWS account to query for the metric, specifying `all` will query all accounts that the monitoring account is permitted to query.
    account_id: typing.Optional[str]
    # Metric data aggregations over specified periods of time. For detailed definitions of the statistics supported by CloudWatch, see https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/Statistics-definitions.html.
    statistic: typing.Optional[str]
    # When the metric query type is `metricQueryType` is set to `Query` and the `metricEditorMode` is set to `Builder`, this field is used to build up an object representation of a SQL query.
    sql: typing.Optional['SQLExpression']
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]
    # @deprecated use statistic
    statistics: typing.Optional[list[str]]

    def __init__(self, query_mode: typing.Optional['CloudWatchQueryMode'] = None, metric_query_type: typing.Optional['MetricQueryType'] = None, metric_editor_mode: typing.Optional['MetricEditorMode'] = None, id_val: str = "", alias: typing.Optional[str] = None, label: typing.Optional[str] = None, expression: typing.Optional[str] = None, sql_expression: typing.Optional[str] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, region: str = "", namespace: str = "", metric_name: typing.Optional[str] = None, dimensions: typing.Optional['Dimensions'] = None, match_exact: typing.Optional[bool] = None, period: typing.Optional[str] = None, account_id: typing.Optional[str] = None, statistic: typing.Optional[str] = None, sql: typing.Optional['SQLExpression'] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, statistics: typing.Optional[list[str]] = None):
        self.query_mode = query_mode if query_mode is not None else CloudWatchQueryMode.METRICS
        self.metric_query_type = metric_query_type
        self.metric_editor_mode = metric_editor_mode
        self.id_val = id_val
        self.alias = alias
        self.label = label
        self.expression = expression
        self.sql_expression = sql_expression
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.region = region
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.match_exact = match_exact
        self.period = period
        self.account_id = account_id
        self.statistic = statistic
        self.sql = sql
        self.datasource = datasource
        self.statistics = statistics

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "queryMode": self.query_mode,
            "id": self.id_val,
            "refId": self.ref_id,
            "region": self.region,
            "namespace": self.namespace,
        }
        if self.metric_query_type is not None:
            payload["metricQueryType"] = self.metric_query_type
        if self.metric_editor_mode is not None:
            payload["metricEditorMode"] = self.metric_editor_mode
        if self.alias is not None:
            payload["alias"] = self.alias
        if self.label is not None:
            payload["label"] = self.label
        if self.expression is not None:
            payload["expression"] = self.expression
        if self.sql_expression is not None:
            payload["sqlExpression"] = self.sql_expression
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.metric_name is not None:
            payload["metricName"] = self.metric_name
        if self.dimensions is not None:
            payload["dimensions"] = self.dimensions
        if self.match_exact is not None:
            payload["matchExact"] = self.match_exact
        if self.period is not None:
            payload["period"] = self.period
        if self.account_id is not None:
            payload["accountId"] = self.account_id
        if self.statistic is not None:
            payload["statistic"] = self.statistic
        if self.sql is not None:
            payload["sql"] = self.sql
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.statistics is not None:
            payload["statistics"] = self.statistics
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "queryMode" in data:
            args["query_mode"] = data["queryMode"]
        if "metricQueryType" in data:
            args["metric_query_type"] = data["metricQueryType"]
        if "metricEditorMode" in data:
            args["metric_editor_mode"] = data["metricEditorMode"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "alias" in data:
            args["alias"] = data["alias"]
        if "label" in data:
            args["label"] = data["label"]
        if "expression" in data:
            args["expression"] = data["expression"]
        if "sqlExpression" in data:
            args["sql_expression"] = data["sqlExpression"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "region" in data:
            args["region"] = data["region"]
        if "namespace" in data:
            args["namespace"] = data["namespace"]
        if "metricName" in data:
            args["metric_name"] = data["metricName"]
        if "dimensions" in data:
            args["dimensions"] = {key: data["dimensions"][key] for key in data["dimensions"].keys()}
        if "matchExact" in data:
            args["match_exact"] = data["matchExact"]
        if "period" in data:
            args["period"] = data["period"]
        if "accountId" in data:
            args["account_id"] = data["accountId"]
        if "statistic" in data:
            args["statistic"] = data["statistic"]
        if "sql" in data:
            args["sql"] = SQLExpression.from_json(data["sql"])
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "statistics" in data:
            args["statistics"] = data["statistics"]        

        return cls(**args)


class CloudWatchQueryMode(enum.StrEnum):
    METRICS = "Metrics"
    LOGS = "Logs"
    ANNOTATIONS = "Annotations"


class MetricQueryType(enum.IntEnum):
    SEARCH = 0
    QUERY = 1


class MetricEditorMode(enum.IntEnum):
    BUILDER = 0
    CODE = 1


class SQLExpression:
    # SELECT part of the SQL expression
    select: typing.Optional['QueryEditorFunctionExpression']
    # FROM part of the SQL expression
    from_val: typing.Optional[typing.Union['QueryEditorPropertyExpression', 'QueryEditorFunctionExpression']]
    # WHERE part of the SQL expression
    where: typing.Optional['QueryEditorArrayExpression']
    # GROUP BY part of the SQL expression
    group_by: typing.Optional['QueryEditorArrayExpression']
    # ORDER BY part of the SQL expression
    order_by: typing.Optional['QueryEditorFunctionExpression']
    # The sort order of the SQL expression, `ASC` or `DESC`
    order_by_direction: typing.Optional[str]
    # LIMIT part of the SQL expression
    limit: typing.Optional[int]

    def __init__(self, select: typing.Optional['QueryEditorFunctionExpression'] = None, from_val: typing.Optional[typing.Union['QueryEditorPropertyExpression', 'QueryEditorFunctionExpression']] = None, where: typing.Optional['QueryEditorArrayExpression'] = None, group_by: typing.Optional['QueryEditorArrayExpression'] = None, order_by: typing.Optional['QueryEditorFunctionExpression'] = None, order_by_direction: typing.Optional[str] = None, limit: typing.Optional[int] = None):
        self.select = select
        self.from_val = from_val
        self.where = where
        self.group_by = group_by
        self.order_by = order_by
        self.order_by_direction = order_by_direction
        self.limit = limit

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.select is not None:
            payload["select"] = self.select
        if self.from_val is not None:
            payload["from"] = self.from_val
        if self.where is not None:
            payload["where"] = self.where
        if self.group_by is not None:
            payload["groupBy"] = self.group_by
        if self.order_by is not None:
            payload["orderBy"] = self.order_by
        if self.order_by_direction is not None:
            payload["orderByDirection"] = self.order_by_direction
        if self.limit is not None:
            payload["limit"] = self.limit
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "select" in data:
            args["select"] = QueryEditorFunctionExpression.from_json(data["select"])
        if "from" in data:
            decoding_map_from_union: dict[str, typing.Union[typing.Type[QueryEditorFunctionExpression], typing.Type[QueryEditorPropertyExpression]]] = {"function": QueryEditorFunctionExpression, "property": QueryEditorPropertyExpression}
            args["from_val"] = decoding_map_from_union[data["from"]["type"]].from_json(data["from"])
        if "where" in data:
            args["where"] = QueryEditorArrayExpression.from_json(data["where"])
        if "groupBy" in data:
            args["group_by"] = QueryEditorArrayExpression.from_json(data["groupBy"])
        if "orderBy" in data:
            args["order_by"] = QueryEditorFunctionExpression.from_json(data["orderBy"])
        if "orderByDirection" in data:
            args["order_by_direction"] = data["orderByDirection"]
        if "limit" in data:
            args["limit"] = data["limit"]        

        return cls(**args)


class QueryEditorFunctionExpression:
    type_val: str
    name: typing.Optional[str]
    parameters: typing.Optional[list['QueryEditorFunctionParameterExpression']]

    def __init__(self, name: typing.Optional[str] = None, parameters: typing.Optional[list['QueryEditorFunctionParameterExpression']] = None):
        self.type_val = QueryEditorExpressionType.FUNCTION
        self.name = name
        self.parameters = parameters

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.name is not None:
            payload["name"] = self.name
        if self.parameters is not None:
            payload["parameters"] = self.parameters
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "parameters" in data:
            args["parameters"] = [QueryEditorFunctionParameterExpression.from_json(item) for item in data["parameters"]]        

        return cls(**args)


class QueryEditorExpressionType(enum.StrEnum):
    PROPERTY = "property"
    OPERATOR = "operator"
    OR = "or"
    AND = "and"
    GROUP_BY = "groupBy"
    FUNCTION = "function"
    FUNCTION_PARAMETER = "functionParameter"


class QueryEditorFunctionParameterExpression:
    type_val: str
    name: typing.Optional[str]

    def __init__(self, name: typing.Optional[str] = None):
        self.type_val = QueryEditorExpressionType.FUNCTION_PARAMETER
        self.name = name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.name is not None:
            payload["name"] = self.name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]        

        return cls(**args)


class QueryEditorPropertyExpression:
    type_val: str
    property_val: 'QueryEditorProperty'

    def __init__(self, property_val: typing.Optional['QueryEditorProperty'] = None):
        self.type_val = QueryEditorExpressionType.PROPERTY
        self.property_val = property_val if property_val is not None else QueryEditorProperty()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "property": self.property_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "property" in data:
            args["property_val"] = QueryEditorProperty.from_json(data["property"])        

        return cls(**args)


class QueryEditorProperty:
    type_val: 'QueryEditorPropertyType'
    name: typing.Optional[str]

    def __init__(self, type_val: typing.Optional['QueryEditorPropertyType'] = None, name: typing.Optional[str] = None):
        self.type_val = type_val if type_val is not None else QueryEditorPropertyType.STRING
        self.name = name

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.name is not None:
            payload["name"] = self.name
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "name" in data:
            args["name"] = data["name"]        

        return cls(**args)


class QueryEditorPropertyType(enum.StrEnum):
    STRING = "string"


class QueryEditorArrayExpression:
    type_val: typing.Literal["and", "or"]
    expressions: list['QueryEditorExpression']

    def __init__(self, type_val: typing.Optional[typing.Literal["and", "or"]] = None, expressions: typing.Optional[list['QueryEditorExpression']] = None):
        self.type_val = type_val if type_val is not None else "and"
        self.expressions = expressions if expressions is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "expressions": self.expressions,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "expressions" in data:
            decoding_map_expressions_array_ref_union: dict[str, typing.Union[typing.Type[QueryEditorArrayExpression], typing.Type[QueryEditorFunctionExpression], typing.Type[QueryEditorFunctionParameterExpression], typing.Type[QueryEditorGroupByExpression], typing.Type[QueryEditorOperatorExpression], typing.Type[QueryEditorArrayExpression], typing.Type[QueryEditorPropertyExpression]]] = {"and": QueryEditorArrayExpression, "function": QueryEditorFunctionExpression, "functionParameter": QueryEditorFunctionParameterExpression, "groupBy": QueryEditorGroupByExpression, "operator": QueryEditorOperatorExpression, "or": QueryEditorArrayExpression, "property": QueryEditorPropertyExpression}
            args["expressions"] = [decoding_map_expressions_array_ref_union[item["type"]].from_json(item) for item in data["expressions"]]        

        return cls(**args)


QueryEditorExpression: typing.TypeAlias = typing.Union['QueryEditorArrayExpression', 'QueryEditorPropertyExpression', 'QueryEditorGroupByExpression', 'QueryEditorFunctionExpression', 'QueryEditorFunctionParameterExpression', 'QueryEditorOperatorExpression']


class QueryEditorGroupByExpression:
    type_val: str
    property_val: 'QueryEditorProperty'

    def __init__(self, property_val: typing.Optional['QueryEditorProperty'] = None):
        self.type_val = QueryEditorExpressionType.GROUP_BY
        self.property_val = property_val if property_val is not None else QueryEditorProperty()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "property": self.property_val,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "property" in data:
            args["property_val"] = QueryEditorProperty.from_json(data["property"])        

        return cls(**args)


class QueryEditorOperatorExpression:
    type_val: str
    property_val: 'QueryEditorProperty'
    # TS type is operator: QueryEditorOperator<QueryEditorOperatorValueType>, extended in veneer
    operator: 'QueryEditorOperator'

    def __init__(self, property_val: typing.Optional['QueryEditorProperty'] = None, operator: typing.Optional['QueryEditorOperator'] = None):
        self.type_val = QueryEditorExpressionType.OPERATOR
        self.property_val = property_val if property_val is not None else QueryEditorProperty()
        self.operator = operator if operator is not None else QueryEditorOperator()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "property": self.property_val,
            "operator": self.operator,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "property" in data:
            args["property_val"] = QueryEditorProperty.from_json(data["property"])
        if "operator" in data:
            args["operator"] = QueryEditorOperator.from_json(data["operator"])        

        return cls(**args)


class QueryEditorOperator:
    """
    TS type is QueryEditorOperator<T extends QueryEditorOperatorValueType>, extended in veneer
    """

    name: typing.Optional[str]
    value: typing.Optional[typing.Union[str, bool, int, list['QueryEditorOperatorType']]]

    def __init__(self, name: typing.Optional[str] = None, value: typing.Optional[typing.Union[str, bool, int, list['QueryEditorOperatorType']]] = None):
        self.name = name
        self.value = value

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.name is not None:
            payload["name"] = self.name
        if self.value is not None:
            payload["value"] = self.value
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "value" in data:
            args["value"] = data["value"]        

        return cls(**args)


QueryEditorOperatorType: typing.TypeAlias = typing.Union[str, bool, int]


QueryEditorOperatorValueType: typing.TypeAlias = typing.Union[str, bool, int, list['QueryEditorOperatorType']]


class CloudWatchLogsQuery(cogvariants.Dataquery):
    """
    Shape of a CloudWatch Logs query
    """

    # Whether a query is a Metrics, Logs, or Annotations query
    query_mode: 'CloudWatchQueryMode'
    id_val: str
    # AWS region to query for the logs
    region: str
    # The CloudWatch Logs Insights query to execute
    expression: typing.Optional[str]
    # Fields to group the results by, this field is automatically populated whenever the query is updated
    stats_groups: typing.Optional[list[str]]
    # Log groups to query
    log_groups: typing.Optional[list['LogGroup']]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # @deprecated use logGroups
    log_group_names: typing.Optional[list[str]]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]

    def __init__(self, query_mode: typing.Optional['CloudWatchQueryMode'] = None, id_val: str = "", region: str = "", expression: typing.Optional[str] = None, stats_groups: typing.Optional[list[str]] = None, log_groups: typing.Optional[list['LogGroup']] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, log_group_names: typing.Optional[list[str]] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None):
        self.query_mode = query_mode if query_mode is not None else CloudWatchQueryMode.LOGS
        self.id_val = id_val
        self.region = region
        self.expression = expression
        self.stats_groups = stats_groups
        self.log_groups = log_groups
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.log_group_names = log_group_names
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "queryMode": self.query_mode,
            "id": self.id_val,
            "region": self.region,
            "refId": self.ref_id,
        }
        if self.expression is not None:
            payload["expression"] = self.expression
        if self.stats_groups is not None:
            payload["statsGroups"] = self.stats_groups
        if self.log_groups is not None:
            payload["logGroups"] = self.log_groups
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.log_group_names is not None:
            payload["logGroupNames"] = self.log_group_names
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "queryMode" in data:
            args["query_mode"] = data["queryMode"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "region" in data:
            args["region"] = data["region"]
        if "expression" in data:
            args["expression"] = data["expression"]
        if "statsGroups" in data:
            args["stats_groups"] = data["statsGroups"]
        if "logGroups" in data:
            args["log_groups"] = [LogGroup.from_json(item) for item in data["logGroups"]]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "logGroupNames" in data:
            args["log_group_names"] = data["logGroupNames"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])        

        return cls(**args)


class LogGroup:
    # ARN of the log group
    arn: str
    # Name of the log group
    name: str
    # AccountId of the log group
    account_id: typing.Optional[str]
    # Label of the log group
    account_label: typing.Optional[str]

    def __init__(self, arn: str = "", name: str = "", account_id: typing.Optional[str] = None, account_label: typing.Optional[str] = None):
        self.arn = arn
        self.name = name
        self.account_id = account_id
        self.account_label = account_label

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "arn": self.arn,
            "name": self.name,
        }
        if self.account_id is not None:
            payload["accountId"] = self.account_id
        if self.account_label is not None:
            payload["accountLabel"] = self.account_label
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "arn" in data:
            args["arn"] = data["arn"]
        if "name" in data:
            args["name"] = data["name"]
        if "accountId" in data:
            args["account_id"] = data["accountId"]
        if "accountLabel" in data:
            args["account_label"] = data["accountLabel"]        

        return cls(**args)


class CloudWatchAnnotationQuery(cogvariants.Dataquery):
    """
    Shape of a CloudWatch Annotation query
    TS type is CloudWatchDefaultQuery = Omit<CloudWatchLogsQuery, 'queryMode'> & CloudWatchMetricsQuery, declared in veneer
    #CloudWatchDefaultQuery: #CloudWatchLogsQuery & #CloudWatchMetricsQuery @cuetsy(kind="type")
    """

    # Whether a query is a Metrics, Logs, or Annotations query
    query_mode: 'CloudWatchQueryMode'
    # Enable matching on the prefix of the action name or alarm name, specify the prefixes with actionPrefix and/or alarmNamePrefix
    prefix_matching: typing.Optional[bool]
    # Use this parameter to filter the results of the operation to only those alarms
    # that use a certain alarm action. For example, you could specify the ARN of
    # an SNS topic to find all alarms that send notifications to that topic.
    # e.g. `arn:aws:sns:us-east-1:123456789012:my-app-` would match `arn:aws:sns:us-east-1:123456789012:my-app-action`
    # but not match `arn:aws:sns:us-east-1:123456789012:your-app-action`
    action_prefix: typing.Optional[str]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # AWS region to query for the metric
    region: str
    # A namespace is a container for CloudWatch metrics. Metrics in different namespaces are isolated from each other, so that metrics from different applications are not mistakenly aggregated into the same statistics. For example, Amazon EC2 uses the AWS/EC2 namespace.
    namespace: str
    # Name of the metric
    metric_name: typing.Optional[str]
    # The dimensions of the metric
    dimensions: typing.Optional['Dimensions']
    # Only show metrics that exactly match all defined dimension names.
    match_exact: typing.Optional[bool]
    # The length of time associated with a specific Amazon CloudWatch statistic. Can be specified by a number of seconds, 'auto', or as a duration string e.g. '15m' being 15 minutes
    period: typing.Optional[str]
    # The ID of the AWS account to query for the metric, specifying `all` will query all accounts that the monitoring account is permitted to query.
    account_id: typing.Optional[str]
    # Metric data aggregations over specified periods of time. For detailed definitions of the statistics supported by CloudWatch, see https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/Statistics-definitions.html.
    statistic: typing.Optional[str]
    # An alarm name prefix. If you specify this parameter, you receive information
    # about all alarms that have names that start with this prefix.
    # e.g. `my-team-service-` would match `my-team-service-high-cpu` but not match `your-team-service-high-cpu`
    alarm_name_prefix: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]
    # @deprecated use statistic
    statistics: typing.Optional[list[str]]

    def __init__(self, query_mode: typing.Optional['CloudWatchQueryMode'] = None, prefix_matching: typing.Optional[bool] = None, action_prefix: typing.Optional[str] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, region: str = "", namespace: str = "", metric_name: typing.Optional[str] = None, dimensions: typing.Optional['Dimensions'] = None, match_exact: typing.Optional[bool] = None, period: typing.Optional[str] = None, account_id: typing.Optional[str] = None, statistic: typing.Optional[str] = None, alarm_name_prefix: typing.Optional[str] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, statistics: typing.Optional[list[str]] = None):
        self.query_mode = query_mode if query_mode is not None else CloudWatchQueryMode.ANNOTATIONS
        self.prefix_matching = prefix_matching
        self.action_prefix = action_prefix
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.region = region
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.match_exact = match_exact
        self.period = period
        self.account_id = account_id
        self.statistic = statistic
        self.alarm_name_prefix = alarm_name_prefix
        self.datasource = datasource
        self.statistics = statistics

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "queryMode": self.query_mode,
            "refId": self.ref_id,
            "region": self.region,
            "namespace": self.namespace,
        }
        if self.prefix_matching is not None:
            payload["prefixMatching"] = self.prefix_matching
        if self.action_prefix is not None:
            payload["actionPrefix"] = self.action_prefix
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.metric_name is not None:
            payload["metricName"] = self.metric_name
        if self.dimensions is not None:
            payload["dimensions"] = self.dimensions
        if self.match_exact is not None:
            payload["matchExact"] = self.match_exact
        if self.period is not None:
            payload["period"] = self.period
        if self.account_id is not None:
            payload["accountId"] = self.account_id
        if self.statistic is not None:
            payload["statistic"] = self.statistic
        if self.alarm_name_prefix is not None:
            payload["alarmNamePrefix"] = self.alarm_name_prefix
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.statistics is not None:
            payload["statistics"] = self.statistics
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "queryMode" in data:
            args["query_mode"] = data["queryMode"]
        if "prefixMatching" in data:
            args["prefix_matching"] = data["prefixMatching"]
        if "actionPrefix" in data:
            args["action_prefix"] = data["actionPrefix"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "region" in data:
            args["region"] = data["region"]
        if "namespace" in data:
            args["namespace"] = data["namespace"]
        if "metricName" in data:
            args["metric_name"] = data["metricName"]
        if "dimensions" in data:
            args["dimensions"] = {key: data["dimensions"][key] for key in data["dimensions"].keys()}
        if "matchExact" in data:
            args["match_exact"] = data["matchExact"]
        if "period" in data:
            args["period"] = data["period"]
        if "accountId" in data:
            args["account_id"] = data["accountId"]
        if "statistic" in data:
            args["statistic"] = data["statistic"]
        if "alarmNamePrefix" in data:
            args["alarm_name_prefix"] = data["alarmNamePrefix"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "statistics" in data:
            args["statistics"] = data["statistics"]        

        return cls(**args)


CloudWatchQuery: typing.TypeAlias = typing.Union['CloudWatchMetricsQuery', 'CloudWatchLogsQuery', 'CloudWatchAnnotationQuery']





def variant_config() -> cogruntime.DataqueryConfig:
    decoding_map_entrypoint_ref_union: dict[str, typing.Union[typing.Type[CloudWatchAnnotationQuery], typing.Type[CloudWatchLogsQuery], typing.Type[CloudWatchMetricsQuery]]] = {"Annotations": CloudWatchAnnotationQuery, "Logs": CloudWatchLogsQuery, "Metrics": CloudWatchMetricsQuery}
    return cogruntime.DataqueryConfig(
        identifier="cloudwatch",
        from_json_hook=lambda data: decoding_map_entrypoint_ref_union[data["queryMode"]].from_json(data),
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..models import common
from ..cog import runtime as cogruntime


class Options:
    # Manually select which field from the dataset to represent the x field.
    x_field: typing.Optional[str]
    # Use the color value for a sibling field to color each bar value.
    color_by_field: typing.Optional[str]
    # Controls the orientation of the bar chart, either vertical or horizontal.
    orientation: common.VizOrientation
    # Controls the radius of each bar.
    bar_radius: typing.Optional[float]
    # Controls the rotation of the x axis labels.
    x_tick_label_rotation: int
    # Sets the max length that a label can have before it is truncated.
    x_tick_label_max_length: int
    # Controls the spacing between x axis labels.
    # negative values indicate backwards skipping behavior
    x_tick_label_spacing: typing.Optional[int]
    # Controls whether bars are stacked or not, either normally or in percent mode.
    stacking: common.StackingMode
    # This controls whether values are shown on top or to the left of bars.
    show_value: common.VisibilityMode
    # Controls the width of bars. 1 = Max width, 0 = Min width.
    bar_width: float
    # Controls the width of groups. 1 = max with, 0 = min width.
    group_width: float
    legend: common.VizLegendOptions
    tooltip: common.VizTooltipOptions
    text: typing.Optional[common.VizTextDisplayOptions]
    # Enables mode which highlights the entire bar area and shows tooltip when cursor
    # hovers over highlighted area
    full_highlight: bool

    def __init__(self, x_field: typing.Optional[str] = None, color_by_field: typing.Optional[str] = None, orientation: typing.Optional[common.VizOrientation] = None, bar_radius: typing.Optional[float] = 0, x_tick_label_rotation: int = 0, x_tick_label_max_length: int = 0, x_tick_label_spacing: typing.Optional[int] = 0, stacking: typing.Optional[common.StackingMode] = None, show_value: typing.Optional[common.VisibilityMode] = None, bar_width: float = 0.97, group_width: float = 0.7, legend: typing.Optional[common.VizLegendOptions] = None, tooltip: typing.Optional[common.VizTooltipOptions] = None, text: typing.Optional[common.VizTextDisplayOptions] = None, full_highlight: bool = False):
        self.x_field = x_field
        self.color_by_field = color_by_field
        self.orientation = orientation if orientation is not None else common.VizOrientation.AUTO
        self.bar_radius = bar_radius
        self.x_tick_label_rotation = x_tick_label_rotation
        self.x_tick_label_max_length = x_tick_label_max_length
        self.x_tick_label_spacing = x_tick_label_spacing
        self.stacking = stacking if stacking is not None else common.StackingMode.NONE
        self.show_value = show_value if show_value is not None else common.VisibilityMode.AUTO
        self.bar_width = bar_width
        self.group_width = group_width
        self.legend = legend if legend is not None else common.VizLegendOptions()
        self.tooltip = tooltip if tooltip is not None else common.VizTooltipOptions()
        self.text = text
        self.full_highlight = full_highlight

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "orientation": self.orientation,
            "xTickLabelRotation": self.x_tick_label_rotation,
            "xTickLabelMaxLength": self.x_tick_label_max_length,
            "stacking": self.stacking,
            "showValue": self.show_value,
            "barWidth": self.bar_width,
            "groupWidth": self.group_width,
            "legend": self.legend,
            "tooltip": self.tooltip,
            "fullHighlight": self.full_highlight,
        }
        if self.x_field is not None:
            payload["xField"] = self.x_field
        if self.color_by_field is not None:
            payload["colorByField"] = self.color_by_field
        if self.bar_radius is not None:
            payload["barRadius"] = self.bar_radius
        if self.x_tick_label_spacing is not None:
            payload["xTickLabelSpacing"] = self.x_tick_label_spacing
        if self.text is not None:
            payload["text"] = self.text
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "xField" in data:
            args["x_field"] = data["xField"]
        if "colorByField" in data:
            args["color_by_field"] = data["colorByField"]
        if "orientation" in data:
            args["orientation"] = data["orientation"]
        if "barRadius" in data:
            args["bar_radius"] = data["barRadius"]
        if "xTickLabelRotation" in data:
            args["x_tick_label_rotation"] = data["xTickLabelRotation"]
        if "xTickLabelMaxLength" in data:
            args["x_tick_label_max_length"] = data["xTickLabelMaxLength"]
        if "xTickLabelSpacing" in data:
            args["x_tick_label_spacing"] = data["xTickLabelSpacing"]
        if "stacking" in data:
            args["stacking"] = data["stacking"]
        if "showValue" in data:
            args["show_value"] = data["showValue"]
        if "barWidth" in data:
            args["bar_width"] = data["barWidth"]
        if "groupWidth" in data:
            args["group_width"] = data["groupWidth"]
        if "legend" in data:
            args["legend"] = common.VizLegendOptions.from_json(data["legend"])
        if "tooltip" in data:
            args["tooltip"] = common.VizTooltipOptions.from_json(data["tooltip"])
        if "text" in data:
            args["text"] = common.VizTextDisplayOptions.from_json(data["text"])
        if "fullHighlight" in data:
            args["full_highlight"] = data["fullHighlight"]        

        return cls(**args)


class FieldConfig:
    # Controls line width of the bars.
    line_width: typing.Optional[int]
    # Controls the fill opacity of the bars.
    fill_opacity: typing.Optional[int]
    # Set the mode of the gradient fill. Fill gradient is based on the line color. To change the color, use the standard color scheme field option.
    # Gradient appearance is influenced by the Fill opacity setting.
    gradient_mode: typing.Optional[common.GraphGradientMode]
    axis_placement: typing.Optional[common.AxisPlacement]
    axis_color_mode: typing.Optional[common.AxisColorMode]
    axis_label: typing.Optional[str]
    axis_width: typing.Optional[float]
    axis_soft_min: typing.Optional[float]
    axis_soft_max: typing.Optional[float]
    axis_grid_show: typing.Optional[bool]
    scale_distribution: typing.Optional[common.ScaleDistributionConfig]
    axis_centered_zero: typing.Optional[bool]
    hide_from: typing.Optional[common.HideSeriesConfig]
    # Threshold rendering
    thresholds_style: typing.Optional[common.GraphThresholdsStyleConfig]
    axis_border_show: typing.Optional[bool]

    def __init__(self, line_width: typing.Optional[int] = 1, fill_opacity: typing.Optional[int] = 80, gradient_mode: typing.Optional[common.GraphGradientMode] = None, axis_placement: typing.Optional[common.AxisPlacement] = None, axis_color_mode: typing.Optional[common.AxisColorMode] = None, axis_label: typing.Optional[str] = None, axis_width: typing.Optional[float] = None, axis_soft_min: typing.Optional[float] = None, axis_soft_max: typing.Optional[float] = None, axis_grid_show: typing.Optional[bool] = None, scale_distribution: typing.Optional[common.ScaleDistributionConfig] = None, axis_centered_zero: typing.Optional[bool] = None, hide_from: typing.Optional[common.HideSeriesConfig] = None, thresholds_style: typing.Optional[common.GraphThresholdsStyleConfig] = None, axis_border_show: typing.Optional[bool] = None):
        self.line_width = line_width
        self.fill_opacity = fill_opacity
        self.gradient_mode = gradient_mode if gradient_mode is not None else common.GraphGradientMode.NONE
        self.axis_placement = axis_placement
        self.axis_color_mode = axis_color_mode
        self.axis_label = axis_label
        self.axis_width = axis_width
        self.axis_soft_min = axis_soft_min
        self.axis_soft_max = axis_soft_max
        self.axis_grid_show = axis_grid_show
        self.scale_distribution = scale_distribution
        self.axis_centered_zero = axis_centered_zero
        self.hide_from = hide_from
        self.thresholds_style = thresholds_style
        self.axis_border_show = axis_border_show

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.line_width is not None:
            payload["lineWidth"] = self.line_width
        if self.fill_opacity is not None:
            payload["fillOpacity"] = self.fill_opacity
        if self.gradient_mode is not None:
            payload["gradientMode"] = self.gradient_mode
        if self.axis_placement is not None:
            payload["axisPlacement"] = self.axis_placement
        if self.axis_color_mode is not None:
            payload["axisColorMode"] = self.axis_color_mode
        if self.axis_label is not None:
            payload["axisLabel"] = self.axis_label
        if self.axis_width is not None:
            payload["axisWidth"] = self.axis_width
        if self.axis_soft_min is not None:
            payload["axisSoftMin"] = self.axis_soft_min
        if self.axis_soft_max is not None:
            payload["axisSoftMax"] = self.axis_soft_max
        if self.axis_grid_show is not None:
            payload["axisGridShow"] = self.axis_grid_show
        if self.scale_distribution is not None:
            payload["scaleDistribution"] = self.scale_distribution
        if self.axis_centered_zero is not None:
            payload["axisCenteredZero"] = self.axis_centered_zero
        if self.hide_from is not None:
            payload["hideFrom"] = self.hide_from
        if self.thresholds_style is not None:
            payload["thresholdsStyle"] = self.thresholds_style
        if self.axis_border_show is not None:
            payload["axisBorderShow"] = self.axis_border_show
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "lineWidth" in data:
            args["line_width"] = data["lineWidth"]
        if "fillOpacity" in data:
            args["fill_opacity"] = data["fillOpacity"]
        if "gradientMode" in data:
            args["gradient_mode"] = data["gradientMode"]
        if "axisPlacement" in data:
            args["axis_placement"] = data["axisPlacement"]
        if "axisColorMode" in data:
            args["axis_color_mode"] = data["axisColorMode"]
        if "axisLabel" in data:
            args["axis_label"] = data["axisLabel"]
        if "axisWidth" in data:
            args["axis_width"] = data["axisWidth"]
        if "axisSoftMin" in data:
            args["axis_soft_min"] = data["axisSoftMin"]
        if "axisSoftMax" in data:
            args["axis_soft_max"] = data["axisSoftMax"]
        if "axisGridShow" in data:
            args["axis_grid_show"] = data["axisGridShow"]
        if "scaleDistribution" in data:
            args["scale_distribution"] = common.ScaleDistributionConfig.from_json(data["scaleDistribution"])
        if "axisCenteredZero" in data:
            args["axis_centered_zero"] = data["axisCenteredZero"]
        if "hideFrom" in data:
            args["hide_from"] = common.HideSeriesConfig.from_json(data["hideFrom"])
        if "thresholdsStyle" in data:
            args["thresholds_style"] = common.GraphThresholdsStyleConfig.from_json(data["thresholdsStyle"])
        if "axisBorderShow" in data:
            args["axis_border_show"] = data["axisBorderShow"]        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="barchart",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from dataclasses import dataclass
from typing import Any, Callable, Optional, Self
from . import variants as cogvariants


@dataclass
class DataqueryConfig:
    identifier: str
    from_json_hook: Callable[[dict[str, Any]], cogvariants.Dataquery]


@dataclass
class PanelCfgConfig:
    identifier: str
    options_from_json_hook: Optional[Callable[[dict[str, Any]], Any]] = None
    field_config_from_json_hook: Optional[Callable[[dict[str, Any]], Any]] = None


class Runtime:
    _instance = None
    dataquery_variants: dict[str, DataqueryConfig]
    panelcfg_variants: dict[str, PanelCfgConfig]

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = object.__new__(cls, *args, **kwargs)
            cls.dataquery_variants = {}
            cls.panelcfg_variants = {}

        return cls._instance

    def register_dataquery_variant(self, variant: DataqueryConfig):
        self.dataquery_variants[variant.identifier] = variant

    def register_panelcfg_variant(self, variant: PanelCfgConfig):
        self.panelcfg_variants[variant.identifier] = variant

    def dataquery_from_json(self, data: dict[str, Any], dataquery_type_hint: str) -> cogvariants.Dataquery:
        if dataquery_type_hint == "" and "datasource" in data and "type" in data["datasource"]:
            dataquery_type_hint = data["datasource"]["type"]

        if dataquery_type_hint != "" and dataquery_type_hint in self.dataquery_variants:
            return self.dataquery_variants[dataquery_type_hint].from_json_hook(data)

        # We have no idea what type the dataquery is: use our `UnknownDataquery` bag to not lose data.
        return UnknownDataquery(data)

    def panelcfg_config(self, variant: str) -> Optional[PanelCfgConfig]:
        return self.panelcfg_variants.get(variant, None)


class UnknownDataquery(cogvariants.Dataquery):
    data: dict[str, Any]

    def __init__(self, data: dict[str, Any]):
        self.data = data

    def to_json(self) -> dict[str, object]:
        return self.data

    @classmethod
    def from_json(cls, data: dict[str, Any]) -> Self:
        return cls(data)


def dataquery_from_json(data: dict[str, Any], dataquery_type_hint: str) -> cogvariants.Dataquery:
    return Runtime().dataquery_from_json(data, dataquery_type_hint)


def panelcfg_config(variant: str) -> Optional[PanelCfgConfig]:
    return Runtime().panelcfg_config(variant)


def register_panelcfg_variant(variant: PanelCfgConfig):
    Runtime().register_panelcfg_variant(variant)


def register_dataquery_variant(variant: DataqueryConfig):
    Runtime().register_dataquery_variant(variant)



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import folder


class Folder(cogbuilder.Builder[folder.Folder]):    
    """
    TODO:
    common metadata will soon support setting the parent folder in the metadata
    """
    
    _internal: folder.Folder

    def __init__(self):
        self._internal = folder.Folder()

    def build(self) -> folder.Folder:
        """
        Builds the object.
        """
        return self._internal    
    
    def uid(self, uid: str) -> typing.Self:    
        """
        Unique folder id. (will be k8s name)
        """
            
        self._internal.uid = uid
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Folder title
        """
            
        self._internal.title = title
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Description of the folder.
        """
            
        self._internal.description = description
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
import enum
from ..cog import runtime as cogruntime


class UpdateConfig:
    render: bool
    data_changed: bool
    schema_changed: bool

    def __init__(self, render: bool = False, data_changed: bool = False, schema_changed: bool = False):
        self.render = render
        self.data_changed = data_changed
        self.schema_changed = schema_changed

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "render": self.render,
            "dataChanged": self.data_changed,
            "schemaChanged": self.schema_changed,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "render" in data:
            args["render"] = data["render"]
        if "dataChanged" in data:
            args["data_changed"] = data["dataChanged"]
        if "schemaChanged" in data:
            args["schema_changed"] = data["schemaChanged"]        

        return cls(**args)


class DebugMode(enum.StrEnum):
    RENDER = "render"
    EVENTS = "events"
    CURSOR = "cursor"
    STATE = "State"
    THROW_ERROR = "ThrowError"


class Options:
    mode: 'DebugMode'
    counters: typing.Optional['UpdateConfig']

    def __init__(self, mode: typing.Optional['DebugMode'] = None, counters: typing.Optional['UpdateConfig'] = None):
        self.mode = mode if mode is not None else DebugMode.RENDER
        self.counters = counters

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "mode": self.mode,
        }
        if self.counters is not None:
            payload["counters"] = self.counters
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]
        if "counters" in data:
            args["counters"] = UpdateConfig.from_json(data["counters"])        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="debug",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )

